-- $LastChangedDate: 2008-10-21 19:35:46 -0400 (Tue, 21 Oct 2008) $
-- Updates HUD every tick.
-- See module.txt for documentation and pitfalls.

---------------------
-- Hud (singleton) --
---------------------

Hud = { }

-- Initialize HUD.
function Hud:Init()
	-- Register timer-tick handler.
	-- Don't update HUD too fast.  Give time for the user to read it.
	Timer:Register( self, defs.HUD_TICK_FREQ )

	-- HUD is composed of lines of HudText objects defined by HudText::Args.
	local hudTextArgs = {
		text="",
		orientation="BOTTOM_LEFT",
		fontSize = 28.0,
		x = 16,
		y = 16,
	}

	local alpha = 1.0
	local yi = 32

	self.hudText1 = sim.HudText:new( hudTextArgs )
	hudTextArgs.y = hudTextArgs.y + yi

	self.hudText2 = sim.HudText:new( hudTextArgs )
	hudTextArgs.y = hudTextArgs.y + yi

	self.hudText3 = sim.HudText:new( hudTextArgs )
	hudTextArgs.y = hudTextArgs.y + yi

	self.hudText4 = sim.HudText:new( hudTextArgs )
	hudTextArgs.y = hudTextArgs.y + yi

	self.hudText5 = sim.HudText:new( hudTextArgs )
	self.hudText5:SetColor( 1.0, 1.0, 0.0, alpha )  -- yellow
	hudTextArgs.y = hudTextArgs.y + yi

	self.hudText6 = sim.HudText:new( hudTextArgs )
	hudTextArgs.y = hudTextArgs.y + yi

	self.hudTextOver = sim.HudText:new( { text="", orientation="TOP_LEFT", x=0, y=yi, fontSize = 34.0, } )
	self.hudTextOver:SetColor( 1.0, 0.6, 0.6, 1.0 )
end

-- Update HUD.
function Hud:Tick()
	local temp = ""
	local current = PlayerActor:GetAircraft()
	local physics = current:GetPhysics()
	local posSph = sim.World:WorldVertex2SphereVertex( current:GetPosition() )

	-- Line 1.
	if ( current:IfCrash() ) then
		temp = ""
	else
		local speedKPH = sim.Physics:Speed2KPH( physics:ComputeSpeed() )
		local throttle = current:GetThrottle()
		local compressionScale = 1.0 / sim.World:GetCompressionScale() + 0.01
		local paused = ""
		if ( sim.Program:IfPaused() ) then paused = "[PAUSED]" end
		temp = string.format( "KPH: %3d THROT: %3d%% FPS: %u  [%dX] %s",
							  speedKPH,
						      throttle * 100.0,
						      sim.Window:GetFPS(),
						      compressionScale,
						      paused )
	end
	self.hudText1:SetText( temp )

	-- Line 2.
	if ( current:IfCrash() ) then
		temp = ""
	else
		temp = string.format( "LAT,LON,ALT: %.2f %.2f %.2f",
		                      posSph[LAT],
						      posSph[LON],
						      posSph[ALT] )
	end
	self.hudText2:SetText( temp )

	-- Line 3.
	if ( current:IfCrash() ) then
		temp = ""
	else
		local coturn = ""
		if ( current:IfCoordinatedTurnExecuting() ) then coturn = "  COTURN  " end
		temp = string.format( "DRAG  : %8.2fN WEIGHT: %8.2fN  AoA: %2d %s",
		                      Distance3(physics:ComputeDragForce()),
						      Distance3(physics:ComputeWeightForce()),
						      physics:ComputeAngleOfAttack(),
						      coturn )
	end
	self.hudText3:SetText( temp )

	-- Line 4.
	if ( current:IfCrash() ) then
		temp = ""
	else
		temp = string.format( "THRUST: %8.2fN LIFT  : %8.2fN",
						      Distance3(physics:ComputeThrustForce()),
						      Distance3(physics:ComputeLiftForce()) )
	end
	self.hudText4:SetText( temp )

	-- Line 5.
	if ( current:IfCrash() ) then
		temp = ""
	else
		temp = ""

		if ( current:IfWheelBrakes() ) then
			temp = "WHEEL BRAKES"
		elseif ( current:IfBrakes() ) then
			temp = "AIR BRAKES" 
		end

		if ( physics:IfStall() ) then
			temp = lib.CatWords( temp, "STALL" )
		end

		local replay = PlayerActor:GetReplay()
		if ( replay:IfPlaying() ) then
			temp = lib.CatWords( temp, "REPLAY" )
		elseif ( replay:IfRecording() ) then
			temp = lib.CatWords( temp, "RECORDING REPLAY" )
		end
	end
	self.hudText5:SetText( temp )

	-- Line 6.
	if ( sim.argv2["-jp"] ) then
		temp = string.format( "JOY %05d %05d %05d %05d",
							 sim.Joystick:GetAxisValue( 0 ),
							 sim.Joystick:GetAxisValue( 1 ),
							 sim.Joystick:GetAxisValue( 2 ),
							 sim.Joystick:GetAxisValue( 3 ) )
		self.hudText6:SetText( temp )
	end

	-- "GAME OVER".
	temp = ""
	if ( current:IfCrash() ) then
		local causeOfCrash = current:GetCauseOfCrash()
		if ( causeOfCrash == sim.Craft.eCauseOfCrash_DOWNWARD_TOO_FAST ) then
			temp = lib.CatWords( temp, "** GAME OVER ** [YOU BURIED YOURSELF]" )
		else
			temp = lib.CatWords( temp, "** GAME OVER ** [INSERT QUARTER]" )
		end
	end
	self.hudTextOver:SetText( temp )
end

-- Update the printing of aircraft's name at its 2D projected position.
function Hud:UpdateAircraftText( hudText, aircraft, aircraftType )
	local wv = aircraft:GetPosition()
	local ev = TransformWorld2Eye( wv )
	if ( sim.Math:IF_Z_FACING( ev[ZZ] ) ) then
		-- Facing viewpoint.
		hudText:SetText( sim.Aircraft.names[aircraftType] )
		hudText:SetPosition3D( wv )
	else
		-- Behind viewpoint (culled).
		hudText:SetText( "" )
	end
end

-- Start HUD.
Hud:Init()
