-- $LastChangedDate: 2008-10-20 10:36:48 -0400 (Mon, 20 Oct 2008) $
-- Basic Lua functions.
-- See module.txt for documentation and pitfalls.

lib = {}  -- namespace

-- Produces a metatable (class table) that inherits methods from another one.
-- Examples in module.txt.
function lib.DerivedClass( BaseMT )
	local t = {}
	setmetatable( t, BaseMT )
	BaseMT.__index = BaseMT
	return t
end

-- Produces an "instance table" from a "class table" (metatable).
-- For use in a Class:new() method.
-- Examples in module.txt.
function lib.NewInstance( classTable, argTable )
	-- Construct instance table.
	local instance = argTable or {}
	setmetatable( instance, classTable )
	classTable.__index = classTable
	return instance
end

-- Catenate two words, placing space if necessary.
function lib.CatWords( s1, s2 )
	if ( s1 ~= "" ) then
		return s1 .. " " .. s2
	else
		return s2
	end
end

-- Split fields of a string.
function lib.SplitFields( str, sep )
	-- This regex pattern needs a separator at end of string.
	if ( str[string.len(str)] ~= sep ) then
		str2 = string.format( "%s%s", str, sep )
		return {str2:match((str2:gsub("[^"..sep.."]*"..sep, "([^"..sep.."]*)"..sep)))}
	else
		return {str:match((str:gsub("[^"..sep.."]*"..sep, "([^"..sep.."]*)"..sep)))}
	end
end

-- Find element in table (used as an array), then remove it.
-- NOP if element is absent.
-- Note: For non-array tables, use: t[key] = nil
function lib.TableRemove( t, val )
	local n = table.getn( t )
	for i = 1, n do
		if ( t[i] == val ) then
			table.remove( t, i )
			return
		end
	end
end

-- Rotate table.
function lib.TableRotate( t )
	local n = table.getn( t )
	if ( n > 1 ) then
		local first = t[1]
		table.remove( t, 1 )
		table.insert( t, first )
	end
end
