-- $LastChangedDate: 2008-10-22 19:07:57 -0400 (Wed, 22 Oct 2008) $
-- Startup.
-- See module.txt for documentation and pitfalls.

-- Create a second argv table indexed by strings.
sim.argv2 = { }
for k,v in pairs(sim.argv) do
	sim.argv2[v] = v
end

-- Log.
if ( sim.argv2["-q"] ) then
	function Log( msg ) end
else
	Log = print
end
Log( "[Lua] startup." );

-- Enable step mode.
-- Although a key can toggle step mode, -step fully enables it.
if ( sim.argv2["-step"] ) then
	Log( "Fully enabled step mode (disabled physics/stall)." )
	sim.Physics:Disable()  -- this permanently disables, there is no Enable()
	sim.Physics:EnableStall( false )
end

-- NOP if controlled by trackball.
if ( sim.argv2["-t"] ) then
	return
end

-- Load all Lua scripts.
dofile( sim.Program:GetScriptDir() .. "/lib.lua" )
dofile( sim.Program:GetScriptDir() .. "/gfx.lua" )
dofile( sim.Program:GetScriptDir() .. "/defs.lua" )
dofile( sim.Program:GetScriptDir() .. "/timer.lua" )
dofile( sim.Program:GetScriptDir() .. "/sim.lua" )
dofile( sim.Program:GetScriptDir() .. "/events.lua" )
dofile( sim.Program:GetScriptDir() .. "/replay.lua" )
dofile( sim.Program:GetScriptDir() .. "/game.lua" )  -- before actors
dofile( sim.Program:GetScriptDir() .. "/weapons.lua" )
dofile( sim.Program:GetScriptDir() .. "/actors.lua" )
dofile( sim.Program:GetScriptDir() .. "/views.lua" )
dofile( sim.Program:GetScriptDir() .. "/cmd.lua" )
dofile( sim.Program:GetScriptDir() .. "/keyboard.lua" )
dofile( sim.Program:GetScriptDir() .. "/hud.lua" )
dofile( sim.Program:GetScriptDir() .. "/scene.lua" )
dofile( sim.Program:GetScriptDir() .. "/mission.lua" )

-- Initialize motion blur if requested (can cause artifacts and slowness).
if ( sim.argv2["-motion-blur"] ) then
	sim.FX:EnableMotionBlur( 0.15 )
	Log( "Enabled motion blur." )
end

-- Load mission script.
if ( sim.argv2["-ossim"] ) then
	dofile( sim.Program:GetScriptDir() .. "/mission_ossim.lua" )
else
	dofile( sim.Program:GetScriptDir() .. "/mission_carrier.lua" )
end

-- If step mode, allow free movement of player.
if ( sim.argv2["-step"] ) then
	PlayerActor.aircraft:SetCollidable( false )
end

-- Optimize scene-graph.
sim.World:OptimizeSceneGraph()

-- Joystick state.
Joystick = { }

-- Save state for when program is reset.
Mission:Save()
Mission:Start()
