/*****************************************************************************
 * $LastChangedDate: 2008-05-04 16:06:39 -0400 (Sun, 04 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Functions to clamp values.
 * @remarks
 * This remains in the base module (not math) since Clamp() is used for Array2D.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef BASE_CLAMP_HH
#define BASE_CLAMP_HH 1

namespace base {

/*****************************************************************************
 * Clamp signed integer in range {0,...,limit}.
 *****************************************************************************/
INLINE int
Clamp( int i, int limit )
{
	if ( UX( i < 0 ) )
		return 0;
	else if ( UX( i > limit ) )
		return limit;
	else
		return i;
}

/*****************************************************************************
 * Clamp value to 0xFF.
 * ClampFF(uint) is the fastest.
 *****************************************************************************/
INLINE uint8
ClampFF( uint i )
{
	// gcc 4.x does inline an optimal x86 CMOV instruction.
	if ( EX(i < 0xff) )
		return uint8(i);
	else
		return 0xff;
}

INLINE uint8
ClampFF( int i )
{
	if ( i < 0 )
		return 0;
	else if ( i > 0xff )
		return 0xff;
	else
		return uint8(i);
}

INLINE uint8
ClampFF( fp f )
{
    if ( f >= 1.0f )
        return 0xff;
    else if ( f < 0.0f )
        return 0;
    else
        return uint8( f * 256.0f );
}

/*****************************************************************************
 * Clamp float to within 0.0,...,1.0.
 *****************************************************************************/
INLINE fp
Clamp1( fp f )
{
	if ( UX(f < 0.0f) )
		return 0.0f;
	else if ( UX(f > 1.0f) )
		return 1.0f;
	else
		return f;
}

INLINE fp
Clamp( fp f, fp low = 0.0f, fp high = 1.0f )
{
	if ( UX( f < low ) )
		return low;
	else if ( UX( f > high ) )
		return high;
	else
		return f;
}

/*****************************************************************************
 * Limit uint to {0,...,0xff}.
 *****************************************************************************/
INLINE uint8
ui2ub( uint i )
{
    if ( i <= 0xff )
        return uint8(i);
    else
        return uint8(0xff);
}

/*****************************************************************************
 * Convert float to unsigned byte.
 *****************************************************************************/
INLINE uint8
f2ub( fp f )
{
    if ( f >= 1.0f )
        return 0xff;
    else if ( f < 0.0f )
        return 0;
    else
        return uint8(f * 256.0f);
}

} // namespace base

#endif // BASE_CLAMP_HH
