/******************************************************************************
 * $LastChangedDate: 2008-01-13 00:25:18 -0500 (Sun, 13 Jan 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Debug support (unused because my code is always perfect).
 *//*
 * LEGAL:	COPYRIGHT (C) 2006 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 ******************************************************************************/

#if DEBUG && ! OS_WINDOWS

#define BASE_DEBUG_CC 1
#include <sstream>
#include <sys/time.h>
#include <time.h>
#if __GLIBC__
#include <malloc.h>
#endif
#include "base/module.hh"
#include "base/debug.hh"

namespace base {

/*****************************************************************************
 * Return total amount of bytes allocated.
 *****************************************************************************/
unsigned long
MemUsed( void )
{
#if __GLIBC__
	struct mallinfo meminfo;
	meminfo = mallinfo();
	return meminfo.arena;
#else
    return 0;  // yeh right
#endif
}

/*****************************************************************************
 * Measure the amount of time a function takes.
 *****************************************************************************/
void
MeasureTime( void(*func)(long), long funcArg, const string& funcName )
{
	// ANSI C code that measures the time the process consumed in user-mode.
	// Excludes kernel-mode (doesn't count time spent in video driver).
	int clocks1 = clock();
	func( funcArg );  // call function to be measured
	int clocks2		 = clock();
	int clocksPerSec = CLOCKS_PER_SEC;
	fp  secUser		 = fp(clocks2 - clocks1) / fp(clocksPerSec);
	ostringstream oss;
	oss << "------------------------------------\n"
        << "function: " << funcName << "\n"
	    << "secs (user) = " << secUser << "\n"
        << "------------------------------------\n";
	string result = oss.str();
	CDEBUG << result.c_str();
}

} // namespace base

#endif // DEBUG && ! OS_WINDOWS
