/*****************************************************************************
 * $LastChangedDate: 2008-09-20 10:29:31 -0400 (Sat, 20 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Main header file.
 * @namespace base
 * @brief Library of base code.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef BASE_MODULE_HH
#define BASE_MODULE_HH 1

#include <string>
using namespace std;

//------------------------------------------------------------------------------
// Coding convention for modules:
// - module.hh is the main header files that includes often-used header files.
// - Init() initializes the module (if applicable).
// - Start() starts the execution of a module (if applicable).

namespace base {

////////////////////////////////////////////////////////////////////////////////
/// @brief InitArgs struct is passed to Init() of every module.
///
struct InitArgs
{
	InitArgs( int argc, char** argv, const string& programName )
	: mArgc(argc), mArgv(argv), mProgramName(programName)
	{
	}

	int		mArgc;
	char**	mArgv;
	string	mProgramName;
};

void Init( const InitArgs& initArgs );

void Start( void );

} // namespace base

//------------------------------------------------------------------------------

// Instead of writing "static" which raises concerns about thread-safety.
#define PERSISTENT   static
#define INLINE       static inline
#ifndef INTERN					// tests can predefine to test static functions
#define INTERN       static
#endif
#ifndef INTERNAL
#define INTERNAL     static
#endif
#define CLASS_CONST  static const
#define CLASS_VAR    static
#define CLASS_METHOD static

#if COMPILE_THREADS
// Unless COMPILE_GCC_TLS=0 was defined by make NO_GCC_TLS=1
#	ifndef COMPILE_GCC_TLS
#	define COMPILE_GCC_TLS 1
#	endif
#endif

// STATS and DEBUG are tied somewhat.
#ifndef STATS
#if DEBUG
#define STATS 1
#else
#define STATS 0
#endif
#endif

// CPU/system defines.
#if !defined(CPU_X86) && (defined(i386)||defined(__i386__)||defined(_X86_)||defined(__x86__)||defined(__x86_64__)||defined(__x86_64))
#define CPU_X86 1
#endif

// To declare globals in header files.
#if BASE_MODULE_CC
#define BASE_GLOBAL  // instantiate when .cc is compiled
#else
#define BASE_GLOBAL extern
#endif

#ifdef OS_WINDOWS
#undef uint
#define uint unsigned int
#endif

// Include often-needed header files.
#if DEBUG
#include "base/typesig_defs.hh"
#endif
#include "base/typesig.hh"
#include "base/os.hh"
#include "base/cc.hh"
#include "base/exception.hh"
#include "base/assert.hh"
#include "base/defs.hh"   // general
#include "base/defs2.hh"  // specific
#include "base/debug.hh"
//#include "base/defs_conf.hh"  [2008/04 removed but could be reinstated]
#include "base/funcs.hh"
#if STATS
#include "base/stats.hh"
#endif
#include "base/thread.hh"  // needed in single-thread too
#include "base/types.hh"
#include "base/safe_ptr.hh"
#include "base/shared_ptr.hh"
//#include "base/types2.hh"  // rarely used
#include "base/types_array.hh"
//#include "base/funcs_stl.hh"
#include "base/stream.hh"
#include "base/globals.hh"

#endif // BASE_MODULE_HH
