/*****************************************************************************
 * $LastChangedDate: 2008-08-31 02:29:07 -0400 (Sun, 31 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Compatibility for compiling single-thread.
 * @remarks	This was stripped-down from Version 2.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef BASE_THREAD_SINGLE_HH
#define BASE_THREAD_SINGLE_HH 1

#if COMPILE_THREADS
#error
#endif

namespace base {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  thread.hh  //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define THREAD_CODE( CODE )

INLINE void
THREAD_RUN( void*(*func)(void*), void* funcArg, UNUSED int priority = 0 )
{
	func( funcArg );
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////  thread_types.hh  /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/// @brief Empty class to indicate a fully thread-safe class.
class Threadable { };

/// @brief Empty class to indicate a class having imperfect thread-safety.
class SemiThreadable { };

/// @brief Empty class to indicate a class cannot be executed concurrently.
class NonThreadable { };

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////  thread_per.hh  /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

template<typename T>
class PerThreadPtr
{
PREVENT_COPYING(PerThreadPtr)
public: 
    PerThreadPtr( void )
    {
        mPtr = new T;
    }

	~PerThreadPtr()
	{
		delete mPtr;
	}

	/// @return NULL if key invalid.
    T* operator->( void )
	{
		return mPtr;
	}

	/// @return Reference to object.
    T& operator*( void )
	{
        return *mPtr;
	}

	/// Limited-use.  Explicitly get raw pointer.
    T* PTR( void )
	{
        return mPtr;
	}

private:
    T*	mPtr;
};

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////  thread_lock.hh  /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define THREAD_AUTO_FAST_LOCK
#define THREAD_AUTO_SLOW_LOCK

////////////////////////////////////////////////////////////////////////////
/// @brief NOP lock.
///
class NoLock
{
PREVENT_COPYING(NoLock)
public:
	// This code is guaranteed to have no bugs ;-)

			NoLock( void ) { }
			~NoLock() { }
	void	Lock( void ) { }
	void	Unlock( void ) { }

	/// @brief See AutoLock
	class Auto
	{
	public:
		Auto( UNUSED NoLock* ) { }
		~Auto() { }
	};
};

class RushLock : public NoLock { };
class FastLock : public NoLock { };
class SlowLock : public NoLock { };

// Write curly braces (behind the scenes, forms an "if" clause).
// BUSY_BEGIN {
// } BUSY_END
#define BUSY_BEGIN			{ PERSISTENT int sBusy = 0;			\
                              if ( not sBusy )
#define BUSY_BEGIN_ASSERT	{ PERSISTENT int sBusy = 0;			\
                              ASSERT( not sBusy );				\
                              if ( not sBusy )
#define BUSY_END			}

} // namespace base

#endif // BASE_THREAD_SINGLE_HH
