/*****************************************************************************
 * $LastChangedDate: 2008-10-21 00:18:53 -0400 (Tue, 21 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Special-effects container.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_FX_HH
#define FX_FX_HH 1

#define GET_FX() (fx::FX::GetInstance())

#include "base/timer.hh"

namespace fx {

class ParticleSystemFX;

////////////////////////////////////////////////////////////////////////////////
/// @brief Special-effects container (Singleton).
///
class FX
{
PREVENT_COPYING(FX)
private:
	// Containers of special-effects.
	typedef list<shptr<Object> > Objects;
	typedef list<shptr<ParticleSystemFX> > ParticleSystems;

	FX( void );
	~FX();

private:
	////////////////////////////////////////////////////////////////////////////
	/// @brief Timer-tick.
	///
	class TimerFunctor : public Timer::Functor
	{
	public:
		TimerFunctor( FX& fx ) : mFX(fx) { }
		void operator()( const Milliseconds millisecElapsed );
	private:
		FX&	mFX;
	};

public:
    CLASS_METHOD FX& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new FX;
		return *msInstance;
	}

// Interface for clients:
public:
	void		Reset( void );
	void		MakeExplosion( const WorldVertex& position,
                               const fp radius,
                               const Milliseconds lifetime );

// Restricted to fx module:
public:
	void		AddZombie( shptr<Object> zombie );
	void		AddZombie( shptr<ParticleSystemFX> zombie );

// Private methods:
private:
	void		Tick( const Milliseconds millisecElapsed );
	void		Detach( Objects& objects );  // plural
	void		Detach( ParticleSystems& particleSystems );  // plural

private:
    CLASS_VAR FX*		msInstance;
	shptr<TimerFunctor>	mTimerFunctor;			///< tick for destroying zombies
	Objects				mObjects;				///< container of active special-effects
	Objects				mZombieObjects;			///< AddZombie() adds to this
	ParticleSystems		mParticleSystems;		///< container of active special-effects
	ParticleSystems		mZombieParticleSystems;	///< AddZombie() adds to this
};

} // namespace fx

#endif // FX_FX_HH
