/*****************************************************************************
 * $LastChangedDate: 2008-08-28 23:26:02 -0400 (Thu, 28 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	ParticleSystemFX is a derivative to be contained in the FX container.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define FX_FX_PARTSYS_CC 1  // FX_FX twice
#include "base/module.hh"
#include "base/conf.hh"
using namespace base;
#include "object/module.hh"
using namespace object;
#include "world/module.hh"
using namespace world;
#include "fx/module.hh"
#include "fx/fx.hh"
#include "fx/partsys.hh"
#include "fx/fx_partsys.hh"
#include "fx/defs.hh"

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////  ParticleSystemFX  ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
ParticleSystemFX::ParticleSystemFX( const Milliseconds lifetime,
                                    const StateBits particleStateBits )
:	ParticleSystem(particleStateBits),  // base class
	mTimeOfDeath(base::Timer::GetInstance().GetElapsedTime() + lifetime),
	mZombie(false)
{
ASSERT( lifetime > Milliseconds(0) );

	// NOP.  FX is responsible for attaching and detaching derivative of this.
}

ParticleSystemFX::~ParticleSystemFX( void )
{
	// NOP.  FX is responsible for attaching and detaching derivative of this.
}

/*****************************************************************************
 * Hook the base class Tick() (base has already registered a timer-tick functor).
 * Every tick, decide whether to expire entire particle system.
 * Add particles is a specific action deferred to a derivative.
 *****************************************************************************/
void
ParticleSystemFX::Tick( const Milliseconds millisecElapsed )
{
CHECK_TYPESIG(this,TYPESIG_PARTICLE_SYSTEM);

	// Time to die?
	if ( millisecElapsed > mTimeOfDeath )
	{
		// Turn into a zombie to be destroyed later by the FX singleton.
		if ( not mZombie )  // once
		{
			mZombie = true;
			GET_FX().AddZombie( this );

		#if DEBUG_PARTSYS
			CDEBUG << "ParticleSystemFX turning into zombie" << endl;
		#endif
		}
	}
	else  // alive
	{
		Parent::Tick( millisecElapsed );
	}
}

} // namespace fx
