/*****************************************************************************
 * $LastChangedDate: 2008-08-31 02:29:07 -0400 (Sun, 31 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Explosion particle-system (private implementation of FX class).
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_FX_PARTSYS_EXPLOSION_HH
#define FX_FX_PARTSYS_EXPLOSION_HH 1

#include "fx/partsys.hh"
#include "fx/fx_partsys.hh"

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/// @brief Explosion particle.
///
class ExplosionParticle : public Particle
{
PREVENT_COPYING(ExplosionParticle)
typedef Particle Parent;
public:
					ExplosionParticle( const Sprite::Args& spriteArgs,
                                       const fp explosionRadius );
					~ExplosionParticle();
	virtual void	Tick( const Milliseconds millisecElapsed );

private:
	const fp		mExplosionRadius;	///< radius of entire explosion
	WorldVertex		mStep;				///< direction of particle's movement
	int				mFadeDec;			///< decrements color (fade-out)
	int				mTickCnt;			///< used for sub-tick granularity
	fp				mSpinRadian;		///< +/- roll
};

////////////////////////////////////////////////////////////////////////////////
/// @brief Explosion particle-system (private implementation of FX class).
///
/// Clients should use the interface FX::MakeExplosion().
/// Explosion class is private to FX (FX contains Explosion objects).
///
class ExplosionParticleSystemFX : public ParticleSystemFX
{
PREVENT_COPYING(ExplosionParticleSystemFX)
typedef ParticleSystemFX Parent;
friend class FX;
private:
					ExplosionParticleSystemFX( const WorldVertex& position,
                                               const fp radius,
                                               const Milliseconds lifetime );
					~ExplosionParticleSystemFX();
	virtual void	Tick( const Milliseconds millisecElapsed );
	void			MakeParticles( const uint cnt );

private:
	const WorldVertex			mPosition;			///< center of explosion
	const fp					mRadius;			///< radius of explosion
	const Milliseconds			mTimeOfDeath;		///< time when particle system expires
	Milliseconds				mTimeOfPrevTick;	///< previous tick
	const RefPtr<osg::StateSet>	mStateSet;			///< particles share one stateset
};

} // namespace fx

#endif // FX_FX_PARTSYS_EXPLOSION_HH
