/*****************************************************************************
 * $LastChangedDate: 2008-08-31 02:29:07 -0400 (Sun, 31 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Texture class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define FX_TEXTURE_CC 1
#include "base/module.hh"
using namespace base;
#include "object/module.hh"
using namespace object;
#include "fx/module.hh"
#include "fx/texture.hh"
#include "fx/image_cache.hh"

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  Texture  ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
Texture::Texture( const string& filename )
:   mTextureObject(new osg::Texture2D)
{
ASSERT( not filename.empty() );
	SET_TYPESIG(this,TYPESIG_TEXTURE);

    // Load texture image into texture object.  Might throw C++ exception.
    mTextureObject->setImage( GET_IMAGE_CACHE().LoadImage(filename).get() );
}

Texture::~Texture()
{
	INVALIDATE_TYPESIG(this,TYPESIG_TEXTURE);
}

/*****************************************************************************
 * Create osg::StateSet with new Texture bound to it.
 *****************************************************************************/
RefPtr<osg::StateSet>
Texture::NewTextureAsStateSet( const string& filename )
{
	shptr<Texture> texture = new Texture( filename );
	RefPtr<osg::StateSet> stateSet = new osg::StateSet;
	texture->Bind( *stateSet );
	return stateSet;
}

/*****************************************************************************
 * Attach texture to node.
 *****************************************************************************/
void
Texture::Bind( osg::StateSet& stateset )
{
CHECK_TYPESIG(this,TYPESIG_TEXTURE);
	stateset.setTextureAttributeAndModes( 0, mTextureObject.get(), osg::StateAttribute::ON );
}

} // namespace fx
