/*****************************************************************************
 * $LastChangedDate: 2008-10-20 10:36:48 -0400 (Mon, 20 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Game-related HUD widgets.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef GAME_HUD_HH
#define GAME_HUD_HH 1

#include "base/module.hh"
#include "base/timer.hh"
using namespace base;
#include "hud/module.hh"
using namespace hud;

namespace game {

#define GET_GAME_HUD() (game::Hud::GetInstance())

////////////////////////////////////////////////////////////////////////////////
/// @brief Game-related HUD widgets.
///
class Hud
{
PREVENT_COPYING(Hud)
private:
	Hud( void );
    ~Hud();

public:
	CLASS_METHOD Hud& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new Hud;
		return *msInstance;
	}

	void		AttachTarget( shptr<Object> target, const string& text );
	void		DetachTarget( shptr<Object> target );
	void		SelectTarget( shptr<Object> target, const bool radarLock );

private:
	void		Tick( const Milliseconds millisecElapsed );

private:
	/// @brief Timer-tick.
	///
	class TickFunctor : public base::Timer::Functor
	{
	public:
		void operator()( const Milliseconds millisecElapsed );
	};
	friend class TickFunctor;

private:
	/// @brief Target indicator box (graphical object).
	///
	struct TargetIndicator
	{
		TargetIndicator( shptr<HudBox> hudBox, shptr<HudText> hudText )
		: mHudBox(hudBox), mHudText(hudText)
		{ }

		shptr<HudBox>	mHudBox;
		shptr<HudText>	mHudText;
	};

private:
	CLASS_VAR Hud*		msInstance;
	shptr<TickFunctor>	mTickFunctor;
	shptr<Object>		mDummyTarget;			///< to indicate no selected target
	shptr<Object>		mSelectedTarget;
	typedef map<shptr<Object>,TargetIndicator> TargetIndicatorMap;
	TargetIndicatorMap	mTargetIndicatorMap;	///< maps an Object to a TargetIndicator box
};

} // namespace game

#endif // GAME_HUD_HH
