/*****************************************************************************
 * $LastChangedDate: 2008-09-30 16:12:50 -0400 (Tue, 30 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	HUD.
 * @remarks
 * Arcane magic and kludges:
 * -------------------------
 * - Constant window size:
 *   Indeed, the width,height of the HUD remains constant at 1280,1024.
 *   Intentionally, there is no listener for a window-resize event,
 *   since OSG will automatically scale osgText strings.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define HUD_HUD_CC 1
#include "base/module.hh"
using namespace base;
#include "glue/module.hh"
#include "glue/osg.hh"
using namespace glue;
#include "graph/module.hh"
using namespace graph;
#include "view/module.hh"
using namespace view;
#include "hud/module.hh"
#include "hud/hud_text.hh"
#include "hud/hud_box.hh"
#include "hud/hud.hh"

#include <osg/CameraNode>

namespace hud {

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////  Hud  //////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

Hud* Hud::msInstance;

/*****************************************************************************
 * ctor/dtor.
 * Root node is needed to place the Geodes containing osg::Text objects.
 *****************************************************************************/
Hud::Hud( void )
:	mCamera(new osg::CameraNode),
	mGroup(new osg::Group),
	mHudBoxes()
{
ASSERT_RUN_ONCE;

	// Stateset (such as disabled fog) has no effect on osg::CameraNode,
	// so an intermediate osg::Group is used to hold nodes of the HUD.
	mCamera->addChild( mGroup.get() );
    mGroup->getOrCreateStateSet()->setMode( GL_LIGHTING, osg::StateAttribute::OFF );
    mGroup->getOrCreateStateSet()->setMode( GL_FOG,      osg::StateAttribute::OFF );
    mGroup->getOrCreateStateSet()->setMode( GL_BLEND,    osg::StateAttribute::OFF );

	// Attach to Window (where to render).
	GET_WINDOW().OsgAddCamera( mCamera.get(), Window::eCamera_2D );

	// Set 2D viewport.
	const WidthHeight<int> wh = GET_WINDOW().GetWindowSize();
    mCamera->setViewport( 0, 0, wh.w, wh.h );

    // Set a 2D projection matrix.
    mCamera->setProjectionMatrix( osg::Matrix::ortho2D( HUD_ORTHO_X1,
                                                        HUD_ORTHO_X2,
                                                        HUD_ORTHO_Y1,
                                                        HUD_ORTHO_Y2 ) );
    // Set the view matrix.
    mCamera->setReferenceFrame( osg::Transform::ABSOLUTE_RF );
    mCamera->setViewMatrix( osg::Matrix::identity() );

    // Only clear the depth buffer.
    mCamera->setClearMask( GL_DEPTH_BUFFER_BIT );

    // Draw subgraph after main camera view.
    mCamera->setRenderOrder( osg::CameraNode::POST_RENDER );
}

Hud::~Hud()
{
	// NOP
}

/*****************************************************************************
 * Enable/disable HUD.
 *****************************************************************************/
void
Hud::Enable( const bool enable )
{
	mCamera->setNodeMask( enable ? ~0 : 0 );
}

/*****************************************************************************
 * Attach a node to HUD.
 *****************************************************************************/
void
Hud::Attach( RefPtr<Node> node )
{
    mGroup->addChild( node.get() );
}

/*****************************************************************************
 * Attach a HudBox to HUD.
 *****************************************************************************/
void
Hud::Attach( shptr<HudBox> hudBox )
{
	// HudBox automatically draws itself and it doesn't have any OSG nodes.
	// But that is hidden from clients: the Attach() interface should be used anyway.
	// Although Hud itself does nothing with a HudBox, Hud has to contain them,
	// as Attach() implies containment which keeps shptr from deleting them.
	mHudBoxes.push_back( hudBox );
}

} // namespace hud
