/*****************************************************************************
 * $LastChangedDate: 2008-09-29 09:39:17 -0400 (Mon, 29 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Base class for HUD objects.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define HUD_HUD_OBJECT_CC 1
#include "base/module.hh"
using namespace base;
#include "glue/module.hh"
#include "glue/osg.hh"
using namespace glue;
#include "graph/module.hh"
using namespace graph;
#include "view/module.hh"
using namespace view;
#include "hud/module.hh"
#include "hud/hud_object.hh"

namespace hud {

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////  HudObject  ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
HudObject::HudObject( void )
:	mWindowPostDrawListener(new HudObject::WindowPostDrawListener(*this))
{
	SET_TYPESIG(this,TYPESIG_HUD_OBJECT);

	// Register function to be called after end of entire frame.
	VIEW_EVENTS.mPostDrawAllViews.Listen( mWindowPostDrawListener );
}

HudObject::~HudObject()
{
	// Unregister.
	VIEW_EVENTS.mPostDrawAllViews.Unlisten( mWindowPostDrawListener );

	INVALIDATE_TYPESIG(this,TYPESIG_HUD_OBJECT);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////  HudObject::WindowPostDrawListener  ///////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor.
 *****************************************************************************/
HudObject::WindowPostDrawListener::WindowPostDrawListener( HudObject& hudObject )
:	mHudObject(hudObject)
{
	// NOP
}

/*****************************************************************************
 * Reflect.
 *****************************************************************************/
void
HudObject::WindowPostDrawListener::operator()( Void nothing )
{
CHECK_TYPESIG(&mHudObject,TYPESIG_HUD_OBJECT);

	// ASSUMPTION: only main view needs a HUD.
	mHudObject.Update( *GET_WINDOW().GetMainView() );
}

} // namespace hud
