/*****************************************************************************
 * $LastChangedDate: 2008-09-29 09:39:17 -0400 (Mon, 29 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Base class for HUD objects.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef HUD_HUD_OBJECT_HH
#define HUD_HUD_OBJECT_HH 1

#include "view/module.hh"
using namespace view;

namespace hud {

////////////////////////////////////////////////////////////////////////////
/// @brief Base class for HUD objects.
///
/// This base class registers a "frame update" callback which
/// derivatives can hook by override the virtual HudObject::Update().
///
class HudObject : public Shared
{
PREVENT_COPYING(HudObject)

private:
	/// @brief Called every frame.  Reflects to HudObject::Update().
	///
	class WindowPostDrawListener : public Event<>::ListenerFunctor
	{
	PREVENT_COPYING(WindowPostDrawListener)
	public:
						WindowPostDrawListener( HudObject& hudObject );
		virtual void	operator()( Void nothing );
	private:
		HudObject&		mHudObject;
	};
	friend class WindowPostDrawListener;

protected:
					HudObject( void );
	virtual			~HudObject();
	virtual void	Update( View& view ) = 0;

// Data:
private:	shptr<WindowPostDrawListener>	mWindowPostDrawListener;	///< post-draw callback
public:		DECL_TYPESIG(TYPESIG_HUD_OBJECT)
};

} // namespace hud

#endif // HUD_HUD_OBJECT_HH
