/*****************************************************************************
 * $LastChangedDate: 2008-02-03 13:14:41 -0500 (Sun, 03 Feb 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Keyboard class (system-neutral).
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef INPUT_KEYBOARD_HH
#define INPUT_KEYBOARD_HH 1

namespace input {

////////////////////////////////////////////////////////////////////////////////
/// @brief Keyboard.
///
/// This class handles keyboard events.
/// Events are enqueued into InputQueue.
///
class Keyboard
{
PREVENT_COPYING(Keyboard)
public:
	////////////////////////////////////////////////////////////////////////////////
	/// @brief Keyboard event.
	///
	class Event
	{
	public:
			Event( void )
			: mKey(0), mSpecial(0), mPress(0), mShift(0), mCtrl(0), mAlt(0)
			{ }

			Event( int key, bool special, bool press, bool shift, bool ctrl, bool alt )
			: mKey(key), mSpecial(special), mPress(press), mShift(shift), mCtrl(ctrl), mAlt(alt)
			{ }

			bool IfModifier( void ) const { return mShift || mCtrl || mAlt; }
	public:
		int		mKey;		// which key
		bool	mSpecial;	// true if a special key such as F1, HOME, etc
	    bool    mPress;     // true if key was pressed, else key was released
		bool	mShift;		// true if Shift was down
		bool	mCtrl;		// true if Ctrl was down
		bool	mAlt;		// true if Alt was down
	};

protected:  // only derivative can construct this
                            Keyboard( void );
	virtual                 ~Keyboard();

public:
    CLASS_METHOD Keyboard&  GetInstance( void );  // returns system-specific derivative
	virtual void            Enable( void ) = 0;

private:
    CLASS_VAR Keyboard*     msInstance;  // Singleton
};

} // namespace input

#endif // INPUT_KEYBOARD_HH
