/*****************************************************************************
 * $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "object/module.hh"
#include "object/aircraft.hh"
using namespace object;
#include "program/module.hh"
#include "program/carrier.hh"
using namespace program;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Carrier"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================
// Need a function to check typesig because of macro/template limitations.
#if ! DOXYGEN
INTERN void
CheckTypesigCarrier( shptr<Carrier> obj )
{
    CHECK_TYPESIG(obj,TYPESIG_CARRIER);
}
#endif // ! DOXYGEN

/*****************************************************************************
 * Lua's garbage collector indirectly invokes this C++ destructor.
 *****************************************************************************/
INTERN int 
Carrier_Destroy( lua_State* L ) 
{ 
   try { 

        return DESTROY_USERDATA_SHPTR<Carrier>( L, CheckTypesigCarrier );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Carrier:new( x,y,z|Vec, carrierNum )
 *****************************************************************************/
INTERN int 
Carrier_New( lua_State* L ) 
{ 
   try { 

		const int         carrierNum = POP_INT( L );
		const WorldVertex carrierPos = POP_VECTOR3<WorldVertex>( L );
		shptr<Carrier> carrier = Carrier::MakeCarrier( carrierPos, carrierNum );
		return NEW_USERDATA_SHPTR<Carrier>( L, carrier, BIND_METATABLE_NAME );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Carrier:PlaceAircraft( aircraft, meterX,meterY,meterZ|Vec3 )
 *****************************************************************************/
INTERN int 
Carrier_PlaceAircraft( lua_State* L ) 
{ 
   try { 

		const GeoVertex geoPos   = POP_VECTOR3<GeoVertex>( L );
		shptr<Aircraft> aircraft = POP_USERDATA_AS_SHPTR<Aircraft>( L );
		shptr<Carrier>  carrier  = POP_USERDATA_AS_SHPTR<Carrier>( L );   // self
		CHECK_TYPESIG(aircraft,TYPESIG_AIRCRAFT);
		CHECK_TYPESIG(carrier,TYPESIG_CARRIER);
		carrier->PlaceAircraft( aircraft, geoPos );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Carrier:Rotate( axis, radian )
 *****************************************************************************/
INTERN int 
Carrier_Rotate( lua_State* L ) 
{ 
   try { 

		const fp   radian = POP_FP( L );
		const uint axis   = POP_INT( L );
		shptr<Carrier> carrier = POP_USERDATA_AS_SHPTR<Carrier>( L );
		CHECK_TYPESIG(carrier,TYPESIG_CARRIER);
		carrier->Rotate( axis, radian );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  { "new", Carrier_New },
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  { "__gc", Carrier_Destroy },
  { "PlaceAircraft", Carrier_PlaceAircraft },
  { "Rotate", Carrier_Rotate },
  { NULL, NULL }
};

void
RegisterCarrier( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
