-- $LastChangedDate: 2008-10-05 20:14:15 -0400 (Sun, 05 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "Carrier",
typesig = "TYPESIG_CARRIER",
luaCanInstantiate = true,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "object/module.hh"
#include "object/aircraft.hh"
using namespace object;
#include "program/module.hh"
#include "program/carrier.hh"
using namespace program;
]],

func_New = {
    comments = "Carrier:new( x,y,z|Vec, carrierNum )",
    code = [[
		const int         carrierNum = POP_INT( L );
		const WorldVertex carrierPos = POP_VECTOR3<WorldVertex>( L );
		shptr<Carrier> carrier = Carrier::MakeCarrier( carrierPos, carrierNum );
		return NEW_USERDATA_SHPTR<Carrier>( L, carrier, BIND_METATABLE_NAME );
    ]]
},

func_Destroy = {
    comments = "Lua's garbage collector indirectly invokes this C++ destructor.",
    code = [[
        return DESTROY_USERDATA_SHPTR<Carrier>( L, CheckTypesigCarrier );
    ]]
},

func_Rotate = {
    comments = "Carrier:Rotate( axis, radian )",
    code = [[
		const fp   radian = POP_FP( L );
		const uint axis   = POP_INT( L );
		shptr<Carrier> carrier = POP_USERDATA_AS_SHPTR<Carrier>( L );
		CHECK_TYPESIG(carrier,TYPESIG_CARRIER);
		carrier->Rotate( axis, radian );
    ]]
},

func_PlaceAircraft = {
    comments = "Carrier:PlaceAircraft( aircraft, meterX,meterY,meterZ|Vec3 )",
    code = [[
		const GeoVertex geoPos   = POP_VECTOR3<GeoVertex>( L );
		shptr<Aircraft> aircraft = POP_USERDATA_AS_SHPTR<Aircraft>( L );
		shptr<Carrier>  carrier  = POP_USERDATA_AS_SHPTR<Carrier>( L );   // self
		CHECK_TYPESIG(aircraft,TYPESIG_AIRCRAFT);
		CHECK_TYPESIG(carrier,TYPESIG_CARRIER);
		carrier->PlaceAircraft( aircraft, geoPos );
    ]]
},

}
