/*****************************************************************************
 * $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "object/module.hh"
#include "object/dyna.hh"
using namespace object;
#include "graph/module.hh"
#include "graph/model_cache.hh"
using namespace graph;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Dyna"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================
// Need a function to check typesig because of macro/template limitations.
#if ! DOXYGEN
INTERN void
CheckTypesigDyna( shptr<Dyna> obj )
{
    CHECK_TYPESIG(obj,TYPESIG_DYNA);
}
#endif // ! DOXYGEN

/*****************************************************************************
 * Lua's garbage collector indirectly invokes this C++ destructor.
 *****************************************************************************/
INTERN int 
Dyna_Destroy( lua_State* L ) 
{ 
   try { 

		return DESTROY_USERDATA_SHPTR<Dyna>( L, CheckTypesigDyna );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Vec3 Dyna:GetPosition()
 *****************************************************************************/
INTERN int 
Dyna_GetPosition( lua_State* L ) 
{ 
   try { 

		shptr<Dyna> dyna = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(dyna,TYPESIG_DYNA);
		const WorldVertex pos = dyna->GetPosition();
		PUSH_VECTOR3( L, pos );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Dyna:new( x,y,z|Vec3, modelPathname, scale, ModelCache.LOAD_SHARED|LOAD_COPY )
 *****************************************************************************/
INTERN int 
Dyna_New( lua_State* L ) 
{ 
   try { 

		const bool        loadCopy  = POP_BOOL( L );
		const fp          scale     = POP_FP( L );
		const string      modelName = POP_STRING( L );
		const WorldVertex pos       = POP_VECTOR3<WorldVertex>( L );

		// Load 3D model.
		shptr<Graph> graph = ModelCache::GetInstance().LoadModel( modelName, scale, loadCopy );
		shptr<Dyna>  dyna  = new Dyna( graph, pos );

		return NEW_USERDATA_SHPTR<Dyna>( L, dyna, BIND_METATABLE_NAME );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Dyna:Rotate( axis, radian )
 *****************************************************************************/
INTERN int 
Dyna_Rotate( lua_State* L ) 
{ 
   try { 

		const fp    radian = POP_FP( L );
		const uint  axis   = POP_INT( L );
		shptr<Dyna> dyna = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(dyna,TYPESIG_DYNA);
		dyna->Rotate( axis, radian );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Dyna:SetPosition( x,y,z|Vec3 )
 *****************************************************************************/
INTERN int 
Dyna_SetPosition( lua_State* L ) 
{ 
   try { 

		const WorldVertex pos = POP_VECTOR3<WorldVertex>( L );
		shptr<Dyna> dyna = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(dyna,TYPESIG_DYNA);
		dyna->SetPosition( pos );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  { "new", Dyna_New },
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  { "__gc", Dyna_Destroy },
  { "GetPosition", Dyna_GetPosition },
  { "Rotate", Dyna_Rotate },
  { "SetPosition", Dyna_SetPosition },
  { NULL, NULL }
};

void
RegisterDyna( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
