-- $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "Dyna",
typesig = "TYPESIG_DYNA",
luaCanInstantiate = true,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "object/module.hh"
#include "object/dyna.hh"
using namespace object;
#include "graph/module.hh"
#include "graph/model_cache.hh"
using namespace graph;
]],

func_New = {
    comments = "Dyna:new( x,y,z|Vec3, modelPathname, scale, ModelCache.LOAD_SHARED|LOAD_COPY )",
    code = [[
		const bool        loadCopy  = POP_BOOL( L );
		const fp          scale     = POP_FP( L );
		const string      modelName = POP_STRING( L );
		const WorldVertex pos       = POP_VECTOR3<WorldVertex>( L );

		// Load 3D model.
		shptr<Graph> graph = ModelCache::GetInstance().LoadModel( modelName, scale, loadCopy );
		shptr<Dyna>  dyna  = new Dyna( graph, pos );

		return NEW_USERDATA_SHPTR<Dyna>( L, dyna, BIND_METATABLE_NAME );
    ]]
},

func_Destroy = {
    comments = "Lua's garbage collector indirectly invokes this C++ destructor.",
    code = [[
		return DESTROY_USERDATA_SHPTR<Dyna>( L, CheckTypesigDyna );
    ]]
},

func_SetPosition = {
    comments = "Dyna:SetPosition( x,y,z|Vec3 )",
    code = [[
		const WorldVertex pos = POP_VECTOR3<WorldVertex>( L );
		shptr<Dyna> dyna = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(dyna,TYPESIG_DYNA);
		dyna->SetPosition( pos );
    ]]
},

func_GetPosition = {
    comments = "Vec3 Dyna:GetPosition()",
    code = [[
		shptr<Dyna> dyna = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(dyna,TYPESIG_DYNA);
		const WorldVertex pos = dyna->GetPosition();
		PUSH_VECTOR3( L, pos );
		return 1;  // number of pushed values
    ]]
},

func_Rotate = {
    comments = "Dyna:Rotate( axis, radian )",
    code = [[
		const fp    radian = POP_FP( L );
		const uint  axis   = POP_INT( L );
		shptr<Dyna> dyna = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(dyna,TYPESIG_DYNA);
		dyna->Rotate( axis, radian );
    ]]
},

}
