-- $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "HudBox",
typesig = "TYPESIG_HUD_BOX",
luaCanInstantiate = true,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "hud/module.hh"
#include "hud/hud_box.hh"
using namespace hud;
]],

func_New = {
    comments = "HudBox:new()",
    code = [[
		// Attach to the Hud.
		shptr<HudBox> hudBox = new HudBox;
		Hud::GetInstance().Attach( hudBox );
		return NEW_USERDATA_SHPTR<HudBox>( L, hudBox, BIND_METATABLE_NAME );
    ]]
},

func_Destroy = {
    comments = "Lua's garbage collector indirectly invokes this C++ destructor.",
    code = [[
		return DESTROY_USERDATA_SHPTR<HudBox>( L, CheckTypesigHudBox );
    ]]
},

func_Set2D = {
    comments = "HudBox:Set2D( x1, y1, x2, y2 ) Define box in 2D terms. ",
    code = [[
		const uint y2 = POP_INT( L );
		const uint x2 = POP_INT( L );
		const uint y1 = POP_INT( L );
		const uint x1 = POP_INT( L );
		shptr<HudBox> hudBox = POP_USERDATA_AS_SHPTR<HudBox>( L );
		CHECK_TYPESIG(hudBox,TYPESIG_HUD_BOX);
		hudBox->Set2D( Rect<int>( x1, y1, x2, y2 ) );
    ]]
},

func_Set3D = {
    comments = "HudBox:Set3D( vec3, width, height )\n * Define box in 3D terms.\n * It sets the rectangle in terms of a 3D position and 2D width,height.",
    code = [[
		const uint        height2 = POP_INT( L );
		const uint        width2  = POP_INT( L );
		const WorldVertex pos3    = POP_VECTOR3<WorldVertex>( L );
		shptr<HudBox> hudBox = POP_USERDATA_AS_SHPTR<HudBox>( L );
		CHECK_TYPESIG(hudBox,TYPESIG_HUD_BOX);
		hudBox->Set3D( pos3, width2, height2 );
    ]]
},

func_SetColor = {
    comments = "HudBox:SetColor( r, g, b, a )  (fp,fp,fp,fp)",
    code = [[
		const RGBA color = POP_RGBA( L );
		shptr<HudBox> hudBox = POP_USERDATA_AS_SHPTR<HudBox>( L );
		CHECK_TYPESIG( hudBox, TYPESIG_HUD_BOX );
		hudBox->SetColor( color );
    ]]
},

}
