-- $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "HudText",
typesig = "TYPESIG_HUD_TEXT",
luaCanInstantiate = true,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "hud/module.hh"
using namespace hud;
]],

func_New = {
    comments = "HudText:new( { text='', orientation=, x=, y=, fontSize= } )\n * text and orientation are required.",
    code = [[
		// Special case: Since some table members are optional,
		// a C++ exception could be thrown during stack operations.
		SAVE_STACK( L );

		// Fill a HudText::Args struct with optional members of a Lua table.
		HudText::Args hudTextArgs;

		// .mText
		try
		{
			hudTextArgs.mText = GET_MEMBER_AS_STRING( L, "text" );
		}
		CATCH_LUA_EXCEPTION

		// .mOrientation
		try
		{
			const string orientation = GET_MEMBER_AS_STRING( L, "orientation" );
			if ( orientation == "TOP_LEFT" )
				hudTextArgs.mOrientation = HudText::eOrientation_TOP_LEFT;
			else if ( orientation == "TOP_RIGHT" )
				hudTextArgs.mOrientation = HudText::eOrientation_TOP_RIGHT;
			else if ( orientation == "BOTTOM_LEFT" )
				hudTextArgs.mOrientation = HudText::eOrientation_BOTTOM_LEFT;
			else if ( orientation == "BOTTOM_RIGHT" )
				hudTextArgs.mOrientation = HudText::eOrientation_BOTTOM_RIGHT;
			else if ( orientation == "CENTER" )
				hudTextArgs.mOrientation = HudText::eOrientation_CENTER;
		}
		CATCH_LUA_EXCEPTION

		// .mPos
		try
		{
			hudTextArgs.mPos[XX] = GET_MEMBER_AS_NUMBER( L, "x", OPTIONAL );
			hudTextArgs.mPos[YY] = GET_MEMBER_AS_NUMBER( L, "y", OPTIONAL );
		}
		catch( const LuaException& ) { } // ignore
		RESTORE_STACK( L );

		// .mFontSize
		try
		{
			hudTextArgs.mFontSize = GET_MEMBER_AS_NUMBER( L, "fontSize", OPTIONAL );
		}
		catch( const LuaException& ) { } // ignore
		RESTORE_STACK( L );

		// Construct HudText from HudText::Args.
		shptr<HudText> hudText = new HudText( hudTextArgs );
	
		// Attach to the Hud.
		Hud::GetInstance().Attach( hudText->GetNode() );

		// Pop the passed table arg.
		lua_pop( L, 1 );

		return NEW_USERDATA_SHPTR<HudText>( L, hudText, BIND_METATABLE_NAME );
    ]]
},

func_Destroy = {
    comments = "Lua's garbage collector indirectly invokes this C++ destructor.",
    code = [[
		return DESTROY_USERDATA_SHPTR<HudText>( L, CheckTypesigHudText );
    ]]
},

func_SetText = {
    comments = "HudText:SetText( string )",
    code = [[
		const string text = POP_STRING( L );
		shptr<HudText> hudText = POP_USERDATA_AS_SHPTR<HudText>( L );
		CHECK_TYPESIG( hudText, TYPESIG_HUD_TEXT );
		hudText->SetText( text );
    ]]
},

func_SetColor = {
    comments = "HudText:SetColor( r, g, b, a )  (fp,fp,fp,fp)",
    code = [[
		const RGBA color = POP_RGBA( L );
		shptr<HudText> hudText = POP_USERDATA_AS_SHPTR<HudText>( L );
		CHECK_TYPESIG( hudText, TYPESIG_HUD_TEXT );
		hudText->SetColor( color );
    ]]
},

func_SetPosition3D = {
    comments = "HudText:SetPosition3D( x,y,z|Vec3 )\n * Set the position of a HudText by projecting a 3D coordinate into 2D.",
    code = [[
		const WorldVertex pos3 = POP_VECTOR3<WorldVertex>( L );
		shptr<HudText> hudText = POP_USERDATA_AS_SHPTR<HudText>( L );
		CHECK_TYPESIG( hudText, TYPESIG_HUD_TEXT );
		hudText->SetPosition3D( pos3 );
    ]]
},

func_SetFontSize = {
    comments = "HudText:SetFontSize( fp )",
    code = [[
		const fp fontSize = POP_FP( L );
		shptr<HudText> hudText = POP_USERDATA_AS_SHPTR<HudText>( L );
		CHECK_TYPESIG( hudText, TYPESIG_HUD_TEXT );
		hudText->SetFontSize( fontSize );
    ]]
},

}
