-- $LastChangedDate: 2008-10-14 21:02:53 -0400 (Tue, 14 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "Joystick",
typesig = nil,
luaCanInstantiate = false,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "control/module.hh"
#include "control/joystick.hh"
using namespace control;
]],

--------------------------------------------------------------------------------

moreComments = [[
 * sim.Joystick abstracts control::Joystick.
 * This binding is unusual in that RegisterJoystick() also registers
 * a listener for the joystick button event to propagate to Lua.
 * No "dequeue axis" method is exported to Lua since the control module
 * handles axis events.
]],

moreCode = [[

#if ! DOXYGEN
////////////////////////////////////////////////////////////////////////////////
/// @brief Joystick button handler that propagates to Lua script.
/// This queues and dequeues events.
///
class ListenerJoystickButton : public Event<input::Joystick::Event>::ListenerFunctor
{
public:
	ListenerJoystickButton( void ) : mEvents() { }

	// Get joystick event.  Caller should ignore axis event (just in case).
	input::Joystick::Event
	GetJoyEvent( void )
	{
		input::Joystick::Event event;  // invalid
		if ( not mEvents.empty() )
		{
			event = mEvents.front();  // dequeue
			mEvents.pop_front();
		}
		return event;
	}

private:
	void operator()( input::Joystick::Event event )
	{
	ASSERT( mEvents.size() < 50000 );
		mEvents.push_back( event );  // enqueue
	}

private:
	deque<input::Joystick::Event> mEvents;
};
#endif // ! DOXYGEN

// The joystick button handler.
INTERN shptr<ListenerJoystickButton> sListenerJoystickButton = new ListenerJoystickButton;

]],

-- Expanded inside of Register*().
moreCodeRegister = [[

	// Listen for joystick button events.
	control::Joystick::GetInstance().mEventButton.Listen( sListenerJoystickButton.PTR() );
]],

--------------------------------------------------------------------------------

func_DequeueButton = {
    comments = "table|nil Joystick:DequeueButton()\n * Returns table { buttonNum=int, buttonVal=bool }",
    code = [[
		// Despite the event struct also containing axis members, ignore them.
		// Only propagate the button members to Lua since the control module handles axis events.
		input::Joystick::Event joyEvent = sListenerJoystickButton->GetJoyEvent();
		if ( joyEvent.mButton != input::Joystick::eButton_NONE )
		{
			lua_newtable( L );
			SET_MEMBER_AS_INT(  L, "buttonNum", joyEvent.mButton );
			SET_MEMBER_AS_BOOL( L, "buttonVal", joyEvent.mButtonVal );
		}
		else
		{
			lua_pushnil( L );  // no button event
		}
		return 1;  // number of pushed values
    ]]
},

func_GetAxisValue = {
    comments = "int16 Joystick:GetAxisValue( axis )",
    code = [[
		const int axis = POP_INT( L );
		const int axisVal = control::Joystick::GetInstance().GetAxisValue( axis );
		lua_pushnumber( L, axisVal );
		return 1;  // number of pushed values
    ]]
},

}
