/*****************************************************************************
 * $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks sim.Keyboard abstracts input::InputQueue.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "input/module.hh"
using namespace input;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Keyboard"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * table|nil Keyboard:DequeueKey()
 * Returns table { press=bool, key=int, lkey=int, shift=bool, ctrl=bool, alt=bool }. 
 * lkey means key converted to lower-case.
 *****************************************************************************/
INTERN int 
Keyboard_DequeueKey( lua_State* L ) 
{ 
   try { 

		input::Keyboard::Event keyEvent;
		if ( input::InputQueue::GetInstance().DequeueKeyEvent( keyEvent ) )
		{
			lua_newtable( L );
			SET_MEMBER_AS_BOOL( L, "press", keyEvent.mPress );
			SET_MEMBER_AS_INT(  L, "key",   keyEvent.mKey );
			SET_MEMBER_AS_INT(  L, "lkey",  tolower(keyEvent.mKey) );
			SET_MEMBER_AS_BOOL( L, "shift", keyEvent.mShift );
			SET_MEMBER_AS_BOOL( L, "ctrl",  keyEvent.mCtrl );
			SET_MEMBER_AS_BOOL( L, "alt",   keyEvent.mAlt );
		}
		else
		{
			lua_pushnil( L );  // no event
		}
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  // new,__gc omitted as Lua cannot instantiate this C++ class.
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  // __gc omitted as Lua cannot instantiate this C++ class.
  { "DequeueKey", Keyboard_DequeueKey },
  { NULL, NULL }
};

void
RegisterKeyboard( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
