-- $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "Object",
typesig = "TYPESIG_OBJECT",
luaCanInstantiate = true,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "object/module.hh"
#include "object/object.hh"
using namespace object;
#include "graph/module.hh"
#include "graph/model_cache.hh"
using namespace graph;
]],

func_New = {
    comments = "Object:new( x,y,z|Vec3, modelPathname, scale, ModelCache.LOAD_SHARED|LOAD_COPY )",
    code = [[
		const bool        loadCopy  = POP_BOOL( L );
		const fp          scale     = POP_FP( L );
		const string      modelName = POP_STRING( L );
		const WorldVertex pos       = POP_VECTOR3<WorldVertex>( L );

		// Load 3D model.
		shptr<Graph> graph = ModelCache::GetInstance().LoadModel( modelName, scale, loadCopy );
		shptr<Object> object = new Object( graph, pos );

		return NEW_USERDATA_SHPTR<Object>( L, object, BIND_METATABLE_NAME );
    ]]
},

func_Destroy = {
    comments = "Lua's garbage collector indirectly invokes this C++ destructor.",
    code = [[
		return DESTROY_USERDATA_SHPTR<Object>( L, CheckTypesigObject );
    ]]
},

func_SetPosition = {
    comments = "Object:SetPosition( x,y,z|Vec3 )",
    code = [[
		const WorldVertex pos = POP_VECTOR3<WorldVertex>( L );
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		object->SetPosition( pos );
    ]]
},

func_GetPosition = {
    comments = "Vec3 Object:GetPosition()",
    code = [[
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		const WorldVertex pos = object->GetPosition();
		PUSH_VECTOR3( L, pos );
		return 1;  // number of pushed values
    ]]
},

func_Rotate = {
    comments = "Object:Rotate( axis, radian )",
    code = [[
		const fp   radian = POP_FP( L );
		const uint axis   = POP_INT( L );
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		object->Rotate( axis, radian );
    ]]
},

func_SetCollidable = {
    comments = "Object:SetCollidable( bool )",
    code = [[
		const bool f = POP_BOOL( L );
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		object->SetCollidable( f );
    ]]
},

}
