/*****************************************************************************
 * $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "view/module.hh"
using namespace view;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "View"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================
// Need a function to check typesig because of macro/template limitations.
#if ! DOXYGEN
INTERN void
CheckTypesigView( shptr<View> obj )
{
    CHECK_TYPESIG(obj,TYPESIG_VIEW);
}
#endif // ! DOXYGEN

/*****************************************************************************
 * Lua's garbage collector indirectly invokes this C++ destructor.
 *****************************************************************************/
INTERN int 
View_Destroy( lua_State* L ) 
{ 
   try { 

		return DESTROY_USERDATA_SHPTR<View>( L, CheckTypesigView );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * View:Enable()
 *****************************************************************************/
INTERN int 
View_Enable( lua_State* L ) 
{ 
   try { 

		const bool enable = POP_BOOL( L );

		shptr<View> view = POP_USERDATA_AS_SHPTR<View>( L );
		CHECK_TYPESIG(view,TYPESIG_VIEW);

		view->Enable( enable );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * View:EnableObjectInMainView()
 *****************************************************************************/
INTERN int 
View_EnableObjectInMainView( lua_State* L ) 
{ 
   try { 

		const bool enable = POP_BOOL( L );
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );

		View::EnableObjectInMainView( *object, enable );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * bool View:IfEnabled()
 *****************************************************************************/
INTERN int 
View_IfEnabled( lua_State* L ) 
{ 
   try { 

		shptr<View> view = POP_USERDATA_AS_SHPTR<View>( L );
		CHECK_TYPESIG(view,TYPESIG_VIEW);
		lua_pushboolean( L, view->IfEnabled() );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * View:new( x1,y1,x2,y2 )
 *****************************************************************************/
INTERN int 
View_New( lua_State* L ) 
{ 
   try { 

		Rect<int> rect;
		rect.y2 = POP_INT( L );
		rect.x2 = POP_INT( L );
		rect.y1 = POP_INT( L );
		rect.x1 = POP_INT( L );

		shptr<View> view = new View( ENABLE, rect );

		// And add View to Window for Lua.
		GET_WINDOW().AddView( view );

		return NEW_USERDATA_SHPTR<View>( L, view, BIND_METATABLE_NAME );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * View:SetOutlineColor( RGBA )
 *****************************************************************************/
INTERN int 
View_SetOutlineColor( lua_State* L ) 
{ 
   try { 

		const RGBA outlineColor( POP_RGBA( L ) );
		shptr<View> view = POP_USERDATA_AS_SHPTR<View>( L );
		CHECK_TYPESIG(view,TYPESIG_VIEW);

		view->SetOutlineColor( outlineColor );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * View:SetViewpoint()
 *****************************************************************************/
INTERN int 
View_SetViewpoint( lua_State* L ) 
{ 
   try { 

		shptr<Viewpoint> viewpoint = POP_USERDATA_AS_SHPTR<Viewpoint>( L );
		CHECK_TYPESIG(viewpoint,TYPESIG_VIEWPOINT);

		shptr<View> view = POP_USERDATA_AS_SHPTR<View>( L );
		CHECK_TYPESIG(view,TYPESIG_VIEW);

		view->SetViewpoint( viewpoint );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Vec3 View:TransformWorld2Eye( x,y,z|Vec3 )
 *****************************************************************************/
INTERN int 
View_TransformWorld2Eye( lua_State* L ) 
{ 
   try { 

		const WorldVertex wv = POP_VECTOR3<WorldVertex>( L );

		shptr<View> view = POP_USERDATA_AS_SHPTR<View>( L );
		CHECK_TYPESIG(view,TYPESIG_VIEW);

		const EyeVertex ev = view->TransformWorld2Eye( wv );
		PUSH_VECTOR3( L, ev );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  { "new", View_New },
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  { "__gc", View_Destroy },
  { "Enable", View_Enable },
  { "EnableObjectInMainView", View_EnableObjectInMainView },
  { "IfEnabled", View_IfEnabled },
  { "SetOutlineColor", View_SetOutlineColor },
  { "SetViewpoint", View_SetViewpoint },
  { "TransformWorld2Eye", View_TransformWorld2Eye },
  { NULL, NULL }
};

void
RegisterView( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
