-- $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "World",
typesig = nil,
luaCanInstantiate = false,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "object/module.hh"
#include "object/object.hh"
using namespace object;
#include "world/module.hh"
#include "world/geo.hh"
using namespace world;
]],

func_AttachObject = {
    comments = "World:AttachObject( Object )",
    code = [[
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		gWorld.Attach( object );
    ]]
},

func_DetachObject = {
    comments = "World:DetachObject( Object )",
    code = [[
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		gWorld.Detach( object );
    ]]
},

func_OptimizeSceneGraph = {
    comments = "World:OptimizeSceneGraph()",
    code = [[
		gWorld.OptimizeSceneGraph();
    ]]
},

func_Meters2Sim = {
    comments = "fp World:Meters2Sim( Meter )",
    code = [[
		const Meter meter( POP_FP( L ) );
		const fp sim = Geo::Meters2Sim( meter );
		lua_pushnumber( L, sim );
		return 1;  // number of pushed values
    ]]
},

func_SphereVertex2WorldVertex = {
    comments = "Vec3 World:SphereVertex2WorldVertex( lat,lon,alt|Vec3 )",
    code = [[
		const SphereVertex sv = POP_SPHERE_VERTEX( L );
		const WorldVertex wv = Geo::SphereVertex2WorldVertex( sv );
		PUSH_VECTOR3<WorldVertex>( L, wv );
		return 1;  // number of pushed values
    ]]
},

func_WorldVertex2SphereVertex = {
    comments = "{lat,lon,alt} World:WorldVertex2SphereVertex( Vec3 )",
    code = [[
		const WorldVertex  wv = POP_VECTOR3<WorldVertex>( L );
		const SphereVertex sv = Geo::WorldVertex2SphereVertex( wv );
		PUSH_SPHERE_VERTEX( L, sv );
		return 1;  // number of pushed values
    ]]
},

func_RotateTangential = {
    comments = "World:RotateTangential( Object, lat,lon,alt|Vec3 )\n * Rotate an Object to be level at a point on spherical world.",
    code = [[
		const SphereVertex sv = POP_SPHERE_VERTEX( L );
		shptr<Object> object = POP_USERDATA_AS_SHPTR<Object>( L );
		CHECK_TYPESIG(object,TYPESIG_OBJECT);
		gWorld.RotateTangential( *object, sv );
    ]]
},

func_GetCompressionScale = {
    comments = "fp World:GetCompressionScale()",
    code = [[
		lua_pushnumber( L, gWorld.GetCompressionScale() );
		return 1;  // number of pushed values
    ]]
},

}
