/*****************************************************************************
 * $LastChangedDate: 2008-10-04 00:18:42 -0400 (Sat, 04 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Aircraft specifications.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef PHYSICS_AIRCRAFT_SPECS_HH
#define PHYSICS_AIRCRAFT_SPECS_HH 1

#include "object/module.hh"
#include "object/rotation_list.hh"
using namespace object;

namespace physics {

////////////////////////////////////////////////////////////////////////////////
/// @brief Specifications/characteristics of an Aircraft.
///
class AircraftSpecs
{
COPYABLE(AircraftSpecs)
public:
			AircraftSpecs( void );
	void	ReadSpecsFromConfFile( const string& confTxtFilename );
	enum eEngineType { eEngineType_PROPELLER, eEngineType_JET };

public:
	// Mass:
	Kilogram		mMass;

	// Thrust:
	Newton1			mMaxThrustMag;		///< magnitude
	eEngineType		mEngineType;

	// Drag:
	fp				mDragAofALimit;		///< parameters of drag formula
	Degree			mDragAofADegree;

	// Lift:
	SpeedKPH		mLiftoffSpeed;
	Degree			mMaxAofA;
	fp				mRangeOfLiftFactor;
	fp				mRangeOfLiftFactorSpeed;
	fp				mRangeOfLiftFactorAofA;

	// Turning:
	Degree			mTurnDegreeLow;		///< degree when banked turn is indicated
	Degree			mTurnDegreeHigh;

	// Airframe:
	Meter			mLength;
	Meter			mHeight;
	SpeedKPH		mMaxSpeed;
	SpeedKPH		mStallSpeed;
	Degree			mRollRate;			///< degree/sec
	Degree			mPitchRate;			///< degree/sec
	Degree			mYawRate;			///< degree/sec

	// 3D model:
	string			mModelFile;				///< .ac file typically
	fp				mModelScale;
	GeoVertex		mModelOffset;			///< [METERS] to adjust center of 3D model
	Meter			mModelOffsetGear;		///< [METERS] to adjust altitude so that landing gears sits on runway
	RotationList	mModelRotationList;		///< orientation of 3D model
	bool			mModelLighting;			///< can make an SR-71 jet black
};

} // namespace physics

#endif // PHYSICS_AIRCRAFT_SPECS_HH
