/*****************************************************************************
 * $LastChangedDate: 2008-07-05 14:12:13 -0400 (Sat, 05 Jul 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module configuration.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define PHYSICS_MODULE_CC 1
#include "base/module.hh"
#include "base/misc.hh"
using namespace base;
#include "physics/module.hh"
#include "physics/conf.hh"
#include "physics/physics_control.hh"

namespace physics {

/*****************************************************************************
 * Initialize module.
 *****************************************************************************/
void
Init( const InitArgs& initArgs )
{
    // ORDER-DEPENDENT

	// Load textual conf file into Conf struct.
	const string confTxtFilename = GetConfTxtFilename( initArgs.mProgramName );
	// Not CONF macro since mValid isn't true yet.
	physics::global::gConf.Init( initArgs.mArgc, initArgs.mArgv, confTxtFilename );

	// Parse args.
	const int argc    = initArgs.mArgc;
	char** const argv = initArgs.mArgv;
	for ( int i = 1; i < argc; ++i )
	{
		// Disable physics.
		if ( STREQ( argv[i], "-no-physics" ) )
		{
			PhysicsControl::PermanentlyDisable();
			CmdargDone( i );
			CLOG << "Disabled physics." << endl;
		}

		// Disable aerodynamic stalling.
		if ( STREQ( argv[i], "-no-stall" ) )
		{
			PhysicsControl::EnableStall( false );
			CmdargDone( i );
			CLOG << "Disabled stalling." << endl;
		}
	}
}

/*****************************************************************************
 * Start module.
 *****************************************************************************/
void
Start( void )
{
	// NOP.
}

} // namespace physics
