/*****************************************************************************
 * $LastChangedDate: 2008-10-04 18:55:38 -0400 (Sat, 04 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Splash text in HUD.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define PROGRAM_HUD_CC 1
#include "base/module.hh"
using namespace base;
#include "types/module.hh"
#include "types/rgba_colors.hh"
using namespace types;
#include "hud/module.hh"
using namespace hud;
#include "program/module.hh"
#include "program/hud.hh"

namespace program {

INTERN const fp HUD_COLOR_ALPHA = 1.0f;
INTERN TinyArray<4,RGBA> sSplashGradientColors;
INTERN const Milliseconds SPLASH_FULL_ALPHA_MILLISECONDS( 1500 );
INTERN const Milliseconds SPLASH_FADE_ALPHA_MILLISECONDS( 4500 );

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////  SplashHud  /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

SplashHud* SplashHud::msInstance;

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
SplashHud::SplashHud( void )
:	mHud(Hud::GetInstance()),
	mHudTextSplash(CtorHudTextSplash()),
	mTickFunctor(new TickFunctor)
{
	mHud.Attach( mHudTextSplash->GetNode() );
}

shptr<HudText>
SplashHud::CtorHudTextSplash( void )
{
	HudText::Args hudTextArgs;
	hudTextArgs.mPos[XX] = 16;
	hudTextArgs.mPos[YY] = 16;
	hudTextArgs.mFontSize = 28.0f;

	// Splash line (center).
	hudTextArgs.mText = defs::PROGRAM_NAME_LONG;
	hudTextArgs.mPos[XX] = 0;
	hudTextArgs.mPos[YY] = 128;  // move upward a little bit
	hudTextArgs.mFontSize = 40.0f;
	hudTextArgs.mOrientation = HudText::eOrientation_CENTER;
	shptr<HudText> hudTextSplash = new HudText( hudTextArgs );
	sSplashGradientColors[HudText::eGradientColor_TOP_LEFT]     = \
	sSplashGradientColors[HudText::eGradientColor_TOP_RIGHT]    = RGBA( 0.0f, 0.8f, 1.0f, 1.0f );
	sSplashGradientColors[HudText::eGradientColor_BOTTOM_LEFT]  = \
	sSplashGradientColors[HudText::eGradientColor_BOTTOM_RIGHT] = RGBA( 0.0f, 0.2f, 1.0f, 1.0f );
	hudTextSplash->SetGradientColors4( sSplashGradientColors );
	return hudTextSplash;
}

SplashHud::~SplashHud()
{
	// NOP
}

/*****************************************************************************
 * Start the updating of HUD.
 *****************************************************************************/
void
SplashHud::Start( void )
{
BEGIN_ONCE {
	Timer::GetInstance().Register( mTickFunctor, defs::SPLASH_HUD_TICK_FREQ );
} END_ONCE
}

/*****************************************************************************
 * Pulse.
 *****************************************************************************/
void
SplashHud::TickFunctor::operator()( const Milliseconds millisecElapsed )
{
	// Reflect.
	SplashHud::GetInstance().Update( millisecElapsed );
}

/*****************************************************************************
 * This update handler is called at a high rate.
 *****************************************************************************/
void
SplashHud::Update( const Milliseconds millisecElapsed )
{
	// Splash line.
	// For a few seconds, keep alpha at 1.0.
	// Then for the next few seconds, fade-out the splash.
	uint8 splashAlpha = 0;
	if ( millisecElapsed < SPLASH_FULL_ALPHA_MILLISECONDS )
	{
		splashAlpha = 0xff;
	}
	else if ( millisecElapsed < SPLASH_FADE_ALPHA_MILLISECONDS )
	{
		splashAlpha = uint8( 0xff - 255.0f * fp(millisecElapsed) / fp(SPLASH_FADE_ALPHA_MILLISECONDS) );
	}
	sSplashGradientColors[HudText::eGradientColor_TOP_LEFT][AA]     = \
	sSplashGradientColors[HudText::eGradientColor_TOP_RIGHT][AA]    = \
	sSplashGradientColors[HudText::eGradientColor_BOTTOM_LEFT][AA]  = \
	sSplashGradientColors[HudText::eGradientColor_BOTTOM_RIGHT][AA] = splashAlpha;
	mHudTextSplash->SetGradientColors4( sSplashGradientColors );

	// When done, stop timer-tick.
	if ( splashAlpha == 0 )
	{
		Timer::GetInstance().Unregister( mTickFunctor );
	}
}

} // namespace program
