/*****************************************************************************
 * $LastChangedDate: 2008-10-04 18:55:38 -0400 (Sat, 04 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Splash text in HUD.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef PROGRAM_HUD_HH
#define PROGRAM_HUD_HH 1

#include "hud/module.hh"
using namespace hud;

namespace program {

////////////////////////////////////////////////////////////////////////////////
/// @brief Singleton to update HUD.  Contains HudText objects.
///
/// To activate SplashHud, call Start().
///
class SplashHud
{
PREVENT_COPYING(SplashHud)
private:
	SplashHud( void );
	~SplashHud();

public:
	CLASS_METHOD SplashHud& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new SplashHud;
		return *msInstance;
	}

public:
	void			Start( void );

private:
	shptr<HudText>	CtorHudTextSplash( void );
	void			Update( const Milliseconds millisecElapsed );

private:
	/// @brief Timer-tick.
	///
	class TickFunctor : public base::Timer::Functor
	{
	public:
		void	operator()( const Milliseconds millisecElapsed );
	};
	friend class TickFunctor;

private:
	// Each HudText represents a line of text in HUD.
	CLASS_VAR SplashHud*	msInstance;
	hud::Hud&				mHud;
	shptr<HudText>			mHudTextSplash;
	shptr<TickFunctor>		mTickFunctor;
};

} // namespace program

#endif // PROGRAM_HUD_HH
