/*****************************************************************************
 * $LastChangedDate: 2008-07-05 21:51:02 -0400 (Sat, 05 Jul 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module configuration.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_CONF_HH
#define VIEW_CONF_HH 1

#include "base/conf_txt.hh"

namespace view {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
	////////////////////////////////////////////////////////////////////////////
	/// @brief Template Method to parse fields in .conf file.
	///
	class ModuleConfTxt : public base::ConfTxt
	{
	public:
	    void ProcessField( string& field, ifstream& conf );
	};

public:
			Conf( void );
	void	Init( int argc, char** argv, const string& confTxtFilename );

private:
	void	ParseCmdlineArgs( int argc, char** argv );
	void	ParseCmdlineArgs2( int argc, char** argv );
	void	ReadConfTxt( const string& confTxtFilename );

public:
	bool            mValid;     ///< true if Conf was init/read

	// Cmd-line args:
    // (none)

	// .conf:
	int				mWindowWidth;
	int				mWindowHeight;
	string			mWindowTitle;
	fp				mOtwViewpointOffset;	///< to move up viewpoint, mitigate frustum clipping at low alt
	bool			mWireframe;				///< -wire,-wireframe
	bool			mUseTrackball;			///< for debugging
	bool			mStatsEnabled;			///< controls OSG statistics
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if VIEW_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// VIEW_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(view::global::gConf.mValid);
                               return view::global::gConf; }
#define VIEW_CONF view::GetConf()
#else
#define VIEW_CONF view::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace view

#endif // VIEW_CONF_HH
