/*****************************************************************************
 * $LastChangedDate: 2008-06-22 20:52:09 -0400 (Sun, 22 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	OSG node masks (cull masks).
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_NODE_MASKS_HH
#define VIEW_NODE_MASKS_HH 1

namespace view {
namespace defs {

//------------------------------------------------------------------------------
// OSG node masks (cull masks):
// First, node masks and cull masks are synonyms.
// When an OSG camera traverses the graph, it does a bitwise AND
// of its "cull mask" vs. the "node mask" of every node.
// If the result of AND is zero, then the node and its children
// are culled from the rendering of that camera.
// For example, to make only the main camera render a 3D model,
// assign its node mask with NODE_MASK_MAIN_CAMERA.
// Window::AddView() assigns a cull mask to an osg::Camera (main or non-main).
//------------------------------------------------------------------------------

const osg::Node::NodeMask NODE_MASK_MAIN_CAMERA = (1<<0);
const osg::Node::NodeMask NODE_MASK_AUX_CAMERA  = (1<<1);

} // namespace defs
} // namespace view

#endif // VIEW_NODE_MASKS_HH
