/*****************************************************************************
 * $LastChangedDate: 2008-10-21 00:13:08 -0400 (Tue, 21 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Events.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_EVENTS_HH
#define WORLD_EVENTS_HH 1

#define WORLD_EVENTS world::global::gEvents

#include "base/event.hh"
using namespace base;
#include "object/module.hh"
using namespace object;

namespace world {

////////////////////////////////////////////////////////////////////////////////
/// @brief Events in World container.
///
/// CRITICAL: All modules holding a reference to an Object
/// should listen for the "detach object" event and release their reference.
/// Else zombie objects will accumulate and waste memory (esp. from 3D files).
///
struct Events
{
	Event<shptr<Object> >	mDetachObject;
};

#if WORLD_MODULE_CC
namespace global { Events gEvents; }
#else
namespace global { extern Events gEvents; }
#endif

////////////////////////////////////////////////////////////////////////////////
/// @brief Any container of an Object must listen for detaching of an Object.
///
/// 3D models may consume large amounts of memory.
/// So releasing all references to an Object being detached is critical!
///
/// World::Detach(object) broadcasts this event (a counterpart is in Lua).
/// This convenience macro expands the functor class.
///
/// mDetachObjectListener(new DetachObjectListener(*this))
/// WORLD_EVENTS.mDetachObject.Listen( mDetachObjectListener );
/// WORLD_EVENTS.mDetachObject.Unisten( mDetachObjectListener );
///
#define CLASS_DETACH_OBJECT_LISTENER( OUTER_CLASS, M_OUTER_VAR )					\
	class DetachObjectListener : public Event<shptr<Object> >::ListenerFunctor		\
	{																				\
	PREVENT_COPYING(DetachObjectListener)											\
	public:																			\
		DetachObjectListener( OUTER_CLASS& outer ) : M_OUTER_VAR(outer) { }			\
		virtual void operator()( shptr<Object> object );							\
	private:																		\
		OUTER_CLASS& M_OUTER_VAR;													\
	};																				\
	friend class DetachObjectListener;

} // namespace world

#endif // WORLD_EVENTS_HH
