/*****************************************************************************
 * $LastChangedDate: 2008-04-10 19:14:32 -0400 (Thu, 10 Apr 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Geodesic class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_GEO_HH
#define WORLD_GEO_HH 1

namespace world {

////////////////////////////////////////////////////////////////////////////////
/// @brief Rigid body that can exert a contact force.
///
/// RigidBody is registered with Geo.
///
class RigidBody : public Shared
{
PREVENT_COPYING(RigidBody)
public:
					RigidBody( void ) { }
	virtual			~RigidBody() { }
	virtual bool	IfContactForce( const GeoVertex& geoPosition ) = 0;
};

////////////////////////////////////////////////////////////////////////////////
/// @brief Mediator class representing geodesic data.
///
/// The representation of the simulated world is factored
/// into graphical and geodesic classes: Geo and World.
///
class Geo
{
PREVENT_COPYING(Geo)
private:
	Geo( void ) { }
	~Geo() { }

public:
// Conversion:
	CLASS_METHOD Meter			Sim2Meters( const fp simDistance );
	CLASS_METHOD fp				Meters2Sim( const Meter meter );
	CLASS_METHOD WorldVertex	GeoVertex2WorldVertex( const GeoVertex& gv );
	CLASS_METHOD WorldVertex	SphereVertex2WorldVertex( const SphereVertex& sv );
	CLASS_METHOD GeoVertex		WorldVertex2GeoVertex( const WorldVertex& wv );
	CLASS_METHOD SphereVertex	WorldVertex2SphereVertex( const WorldVertex& wv );

// Physics:
	CLASS_METHOD void			RegisterRigidBody( shptr<RigidBody> rigidBody );
	CLASS_METHOD bool			IfContactForce( const GeoVertex& geoPosition );

private:
	typedef vector<shptr<RigidBody> > RigidBodyList;
	CLASS_VAR RigidBodyList		msRigidBodyList;
};

} // namespace world

#endif // WORLD_GEO_HH
