/*****************************************************************************
 * $LastChangedDate: 2008-09-01 11:11:25 -0400 (Mon, 01 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of basic types (base).
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "base/module.hh"
#include "base/time.hh"
using namespace base;
#include "math/module.hh"  // FP_EQ() used to be in base
using namespace math;

/*****************************************************************************
 * Test Array class.
 *****************************************************************************/
bool
TestArray( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing Array class. \n";
	CDEBUG << "============================================\n";

	Array<int> arr;
	if ( not arr.empty() ) return Fail();
	if ( arr.size() != 0 ) return Fail();

	arr.push_back( 100 );
	if ( arr.empty() ) return Fail();
	if ( arr.size() != 1 ) return Fail();
	if ( arr[0] != 100 ) return Fail();

	arr.resize( 3 );
	if ( arr.empty() ) return Fail();
	if ( arr.size() != 3 ) return Fail();
	if ( arr[0] != 100 ) return Fail();

	Array<int> arr2( 50 );
	if ( arr2.empty() ) return Fail();
	if ( arr2.size() != 50 ) return Fail();
	arr2.push_back( 123456 );
	if ( arr2[50] != 123456 ) return Fail();

	CDEBUG << "ok \n";
	return true;
}

/*****************************************************************************
 * Test TinyArray class.
 *****************************************************************************/
bool
TestTinyArray( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing TinyArray class. \n";
	CDEBUG << "============================================\n";

 {
	CDEBUG << "----------- basic test TinyArray<3> --------- \n";
	TinyArray<3> a;
	a[0] = 1;
	a[1] = 2;
	a[2] = 3;
	if (    a[0] != 1
         || a[1] != 2
         || a[2] != 3 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<3,float> --------- \n";
	TinyArray<2,float> a;
	a[0] = 12.34;
	a[1] = 56.78;

	if ( not FP_EQ<double>( a[0], 12.34 )
      || not FP_EQ<double>( a[1], 56.78 ) ) return Fail();

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<>(val) constructor> --------- \n";
	TinyArray<2> a(123);		// initialize every element with same value
	if (   a[0] != 123
        || a[1] != 123 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<>() = { } constructor> --------- \n";
	int arr[3] = { 64, 32, 16 };
	TinyArray<3> a = arr;
	if (   a[0] != 64
        || a[1] != 32
        || a[2] != 16 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<> copy constructor> --------- \n";
	TinyArray<2> a1;
	a1[0] = 101;
	a1[1] = 202;
	TinyArray<2> a2 = a1;
	if (    a2[0] != 101
         || a2[1] != 202 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<>.Fill(),Cnt()  --------- \n";
	TinyArray<50> a(555);
	a.Fill(777);

	if ( a.GetCnt() != 50 ) return Fail();
	for ( uint i = 0; i < a.GetCnt(); ++i )
		if ( a[i] != 777 ) return Fail();

	CDEBUG << "sizeof( TinyArray<50> ) = " << sizeof(a) << endl;

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<> operator=() --------- \n";
	TinyArray<2> a1(0);
	TinyArray<2> a2(1);
	a1[0] = 112;
	a1[1] = 221;
	a2 = a1;
	if (    a2[0] != 112
         || a2[1] != 221 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<> operator==/!=()  --------- \n";
	TinyArray<2> a1(3);
	TinyArray<2> a2(4);

	a1 = a2;			// they're equal
	if ( ! (a1 == a2) ) return Fail();
	if ( a1 != a2 )  return Fail();

	a1[0] = 99; a2[0] = 100;	// now they're different
	if ( a1 == a2 ) return Fail();
	if ( ! (a1 != a2) ) return Fail();

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray<>.operator<()  --------- \n";
	TinyArray<4> a1(555), a2(555);
	++a2[3];	// slightly larger
	if ( ! (a1 < a2) ) return Fail();
	if (   (a2 < a1) ) return Fail();

	a2[3] = a1[3];
	++a1[0];	// slightly larger
	if (   (a1 < a2 ) ) return Fail();
	if ( ! (a2 < a1 ) ) return Fail();

	a1 = a2;	// equal
	if (   (a1 < a2 ) ) return Fail();
	if (   (a2 < a1 ) ) return Fail();

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test TinyArray invalid index  --------- \n";
	TinyArray<10> ar;
    try
    {
        if ( ar[10000] )
            return Fail();  // fail in either case
        else
            return Fail();

        if ( ar[uint(-1)] )
            return Fail();  // fail in either case
        else
            return Fail();
    }
    catch (...)
    {
        CDEBUG << "caught exception (expected)" << endl;
    }
 }

	return true;
}

/*****************************************************************************
 * Test SmallArray class.
 *****************************************************************************/
bool
TestSmallArray( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing SmallArray class. \n";
	CDEBUG << "============================================\n";

 {
	CDEBUG << "----------- test SmallArray<> --------- \n";

    // Remember that SmallArray is constructed with a fixed-size (template arg),
    // but it has a variable-size which is initially zero.

	SmallArray<10,int> a1;                // 10 is its FIXED-SIZE
    if ( not a1.empty() ) return Fail();  // yes, it's VARIABLE-SIZE should be empty
    if ( a1.size() != 0 ) return Fail();

    a1.resize( 2 );
    if ( a1.empty() ) return Fail();
    if ( a1.size() != 2 ) return Fail();

    a1[0] = 123;
    a1[1] = 456;
    a1.resize( 4 );  // resize() shouldn't destroy/corrupt existing elements
    if ( a1[0] != 123 ) return Fail();
    if ( a1[1] != 456 ) return Fail();

    a1.clear();
    if ( not a1.empty() ) return Fail();
    if ( a1.size() != 0 ) return Fail();

    CDEBUG << "ok" << endl;
 }

 {
	CDEBUG << "----------- test SmallArray<> error cases --------- \n";
	SmallArray<256,long> a1;
    try
    {
    	a1[0] = 123;  // invalid index: array's variable-size is zero!
    }
    catch (...)
    {
        CDEBUG << "caught exception (expected)" << endl;
    }

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SmallArray<> operator=() --------- \n";
	SmallArray<2,long> a1( 2 );  // idiom: fixed-size = variable-size
	SmallArray<2,long> a2( 2 );
	a1[0] = 112;
	a1[1] = 221;
	a2 = a1;
	if ( a2[0] != 112
      || a2[1] != 221 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SmallArray<> operator=() (different var-size) --------- \n";
	SmallArray<3,long> a1;
	SmallArray<3,long> a2;
	a1.push_back( 1000 );  // a1 has 2 elems, a2 is empty
	a1.push_back( 1001 );
	a2 = a1;               // copy
    if ( a1.size() != 2 ) return Fail();
    if ( a2.size() != 2 ) return Fail();
    if ( not (a1 == a2 ) ) return Fail();
	if ( a2[0] != 1000
      || a2[1] != 1001 ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SmallArray<> operator==/!=()  --------- \n";
	SmallArray<2,short> a1( 2 );
	SmallArray<2,short> a2( 2 );

	a1[0] = a2[0] = 123;
	a1[1] = a2[1] = 456;

	a1 = a2;			// they're equal
	if ( ! (a1 == a2) ) return Fail();
	if ( a1 != a2 )  return Fail();

	++a2[0];            // they're different
	if ( a1 == a2 ) return Fail();
	if ( ! (a1 != a2) ) return Fail();

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SmallArray<>.operator<()  --------- \n";

	SmallArray<2,short> a1( 2 );
	SmallArray<2,short> a2( 2 );

	a1[0] = 100;
    a1[1] = 100;
	a2[0] = 100;
    a2[1] = 100;  // they're equal
	if ( a1 < a2 ) return Fail();
	if ( a2 < a1 ) return Fail();

	a1[0] =  99;
    a1[1] = 100;
	a2[0] = 100;
    a2[1] = 100;  // a1 is less
	if ( not (a1 < a2) ) return Fail();
	if ( a2 < a1 ) return Fail();

	a1[0] = 100;
    a1[1] =  99;
	a2[0] = 100;
    a2[1] = 100;  // a1 is less
	if ( not (a1 < a2) ) return Fail();
	if ( a2 < a1 ) return Fail();

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SmallArray STL methods  --------- \n";
	SmallArray<100,int> ar;
    ar.push_back( -1 );
    ar.push_back( -2 );
    if ( ar.size() != 2 ) return Fail();
    if ( ar[0] != -1 ) return Fail();
    if ( ar[1] != -2 ) return Fail();
	CDEBUG << "ok \n";
 }

    return true;
}

/*****************************************************************************
 * Test SortedArray class.
 *****************************************************************************/
bool
TestSortedArray( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing SortedArray class. \n";
	CDEBUG << "============================================\n";

 {
	CDEBUG << "----------- test SortedArray insert()  --------- \n";
    SortedArray<int> ar;

    if ( ar.insert_last( 3 ) != 0 ) return Fail();  // insert() returns index of inserted item
    if ( ar.insert_last( 1 ) != 0 ) return Fail();  // { 1, 3 }
    if ( ar.insert_last( 0 ) != 0 ) return Fail();  // { 0, 1, 3 }
    if ( ar.insert_last( 2 ) != 2 ) return Fail();  // { 0, 1, 2, 3 }

    if ( ar[0] != 0 ) return Fail(); 
    if ( ar[1] != 1 ) return Fail(); 
    if ( ar[2] != 2 ) return Fail(); 
    if ( ar[3] != 3 ) return Fail(); 
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SortedArray insert() [edge cases] --------- \n";
    SortedArray<int> ar;

    ar.insert_last( 300 );
    if ( ar.size() != 1 ) return Fail();
    if ( ar[0] != 300 ) return Fail();

    ar.insert_last( 200 );
    if ( ar.size() != 2 ) return Fail();
    if ( ar[0] != 200 ) return Fail();
    if ( ar[1] != 300 ) return Fail();
 }
 {
    SortedArray<int> ar;
    ar.insert_last( 111 );
    ar.insert_last( 333 );
    ar.insert_last( 222 );
    ar.insert_last( 111 );
    ar.insert_last( 333 );
    if ( ar.size() != 5 ) return Fail();
    if ( ar[0] != 111 ) return Fail(); 
    if ( ar[1] != 111 ) return Fail(); 
    if ( ar[2] != 222 ) return Fail(); 
    if ( ar[3] != 333 ) return Fail(); 
    if ( ar[4] != 333 ) return Fail();

    // Test invalid index along the way.
    try
    {
        if ( ar[0xdead] )
            return Fail();
        else
            return Fail();
    }
    catch (...)
    {
    }
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SortedArray remove_first() --------- \n";
    SortedArray<int> ar;

    ar.insert_last( 10 );
    ar.insert_last( 20 );
    ar.insert_last( 20 );  // catch removing multiple equal values (should remove one item only)
    ar.insert_last( 30 );

	{ pair<bool,uint> removal = ar.remove_first( 20 ); if ( not removal.first ) return Fail(); }
    if ( ar.empty() ) return Fail();
    if ( ar.size() != 3 ) return Fail();
    if ( ar[0] != 10 ) return Fail();
    if ( ar[1] != 20 ) return Fail();
    if ( ar[2] != 30 ) return Fail();

    { pair<bool,uint> removal = ar.remove_first( 30 ); if ( not removal.first ) return Fail(); }
    if ( ar.size() != 2 ) return Fail();
    if ( ar[0] != 10 ) return Fail();
    if ( ar[1] != 20 ) return Fail();

    { pair<bool,uint> removal = ar.remove_first( 20 ); if ( not removal.first ) return Fail(); }
    if ( ar.size() != 1 ) return Fail();

    { pair<bool,uint> removal = ar.remove_first( 10 ); if ( not removal.first ) return Fail(); }
    if ( ar.size() != 0 ) return Fail();
    if ( not ar.empty() ) return Fail();
	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SortedArray remove_first() [edge cases] --------- \n";
    SortedArray<int> ar;

    // Test removing an non-existent element.
    // Should be a NOP.
    ar.insert_last( 33 );
    ar.insert_last( 22 );
    ar.insert_last( 11 );
	{ pair<bool,uint> removal = ar.remove_first( 0xb00bb00b ); if ( removal.first ) return Fail(); }
	{ pair<bool,uint> removal = ar.remove_first( -1         ); if ( removal.first ) return Fail(); }
	{ pair<bool,uint> removal = ar.remove_first( 9999       ); if ( removal.first ) return Fail(); }
	{ pair<bool,uint> removal = ar.remove_first( 1          ); if ( removal.first ) return Fail(); }
	{ pair<bool,uint> removal = ar.remove_first( 12         ); if ( removal.first ) return Fail(); }
	{ pair<bool,uint> removal = ar.remove_first( 34         ); if ( removal.first ) return Fail(); }

    if ( ar.size() != 3 ) return Fail();
    if ( ar[0] != 11 ) return Fail();
    if ( ar[1] != 22 ) return Fail();
    if ( ar[2] != 33 ) return Fail();

    // Test starting from scratch and rebuilding.
    ar.clear();
    if ( ar.size() != 0 ) return Fail();
    { pair<bool,uint> removal = ar.remove_first( 9999 ); if ( removal.first ) return Fail(); }
    { pair<bool,uint> removal = ar.remove_first( -9999 ); if ( removal.first ) return Fail(); }
    ar.insert_last( 100 );
    ar.insert_last( 10 );
    if ( ar.size() != 2 ) return Fail();
    if ( ar[0] != 10 ) return Fail();
    if ( ar[1] != 100 ) return Fail();
    { pair<bool,uint> removal = ar.remove_first( 100 ); if ( not removal.first ) return Fail(); }
    if ( ar.size() != 1 ) return Fail();
    if ( ar[0] != 10 ) return Fail();
    { pair<bool,uint> removal = ar.remove_first( 10 ); if ( not removal.first ) return Fail(); }
    if ( ar.size() != 0 ) return Fail();

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SortedArray remove_idx() --------- \n";
   {
    SortedArray<int> ar;
    ar.insert_last( 111 ); // 0
    ar.insert_last( 222 ); // 1
    ar.insert_last( 333 ); // 2
	ar.remove_idx( 1 );
	if ( ar.size() != 2 ) return Fail();
	if ( ar[0] != 111 ) return Fail();
	if ( ar[1] != 333 ) return Fail();
   }
   {
    SortedArray<int> ar;
    ar.insert_last( 333 ); // 2 because of sorting
    ar.insert_last( 222 ); // 1
    ar.insert_last( 111 ); // 0
	ar.remove_idx( 0 );  // remove 111
	if ( ar.size() != 2 ) return Fail();
	if ( ar[0] != 222 ) return Fail();
	if ( ar[1] != 333 ) return Fail();
	// remove_idx() should ASSERT out-of-range index
	try
	{
		ar.remove_idx( 5 );  // oops
		return Fail();   // ouch
	}
	catch(...)
	{
		CDEBUG << "(caught exception from bad index for SortedArray::remove_idx())" << endl;
	}
   }
 }
    // find_first() was tricky to write and has to handle some special-cases.
    // It needs thorough testing.
 {
	CDEBUG << "----------- test SortedArray find_first() --------- \n";
    // Copy/paste from find_first() ;-)
    SortedArray<int> ar;

    pair<bool,uint> match;  // index is undefined if false, don't check index in that case

    match = ar.find_first( 0 ); if ( match.first ) return Fail();
    match = ar.find_first( -9999 ); if ( match.first ) return Fail();

    if ( ar.insert_last( 111 ) != 0 ) return Fail();
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_first( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 112 ); if ( match.first ) return Fail();  // check what would be a following item

    if ( ar.insert_last( 222 ) != 1 ) return Fail();
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_first( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 112 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_first( 222 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_first( 221 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 223 ); if ( match.first ) return Fail();  // check what would be a following item

    if ( ar.insert_last( 333 ) != 2 ) return Fail();
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_first( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 112 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_first( 222 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_first( 221 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 223 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_first( 333 ); if ( not match.first || match.second != 2 ) return Fail();
    match = ar.find_first( 332 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 334 ); if ( match.first ) return Fail();  // check what would be a following item

    if ( ar.insert_last( 444 ) != 3 ) return Fail();
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_first( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 112 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_first( 222 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_first( 221 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 223 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_first( 333 ); if ( not match.first || match.second != 2 ) return Fail();
    match = ar.find_first( 332 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 334 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_first( 444 ); if ( not match.first || match.second != 3 ) return Fail();
    match = ar.find_first( 443 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_first( 445 ); if ( match.first ) return Fail();  // check what would be a following item

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SortedArray find_last() --------- \n";
    SortedArray<int> ar;

    pair<bool,uint> match;  // index is undefined if false, don't check index in that case

    match = ar.find_last( 0 ); if ( match.first ) return Fail();
    match = ar.find_last( -9999 ); if ( match.first ) return Fail();

    if ( ar.insert_last( 111 ) != 0 ) return Fail();
    match = ar.find_last( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 112 ); if ( match.first ) return Fail();  // check what would be a following item

    if ( ar.insert_last( 222 ) != 1 ) return Fail();
    match = ar.find_last( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 112 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_last( 222 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_last( 221 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 223 ); if ( match.first ) return Fail();  // check what would be a following item

    if ( ar.insert_last( 333 ) != 2 ) return Fail();
    match = ar.find_last( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 112 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_last( 222 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_last( 221 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 223 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_last( 333 ); if ( not match.first || match.second != 2 ) return Fail();
    match = ar.find_last( 332 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 334 ); if ( match.first ) return Fail();  // check what would be a following item

    if ( ar.insert_last( 444 ) != 3 ) return Fail();
    match = ar.find_last( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last( 110 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 112 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_last( 222 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_last( 221 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 223 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_last( 333 ); if ( not match.first || match.second != 2 ) return Fail();
    match = ar.find_last( 332 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 334 ); if ( match.first ) return Fail();  // check what would be a following item
    match = ar.find_last( 444 ); if ( not match.first || match.second != 3 ) return Fail();
    match = ar.find_last( 443 ); if ( match.first ) return Fail();  // check what would be a preceding item
    match = ar.find_last( 445 ); if ( match.first ) return Fail();  // check what would be a following item

	CDEBUG << "ok \n";
 }

 {
	CDEBUG << "----------- test SortedArray find_first/last() equal range cases --------- \n";
    SortedArray<int> ar;

    // Along the way, try finding non-existent items (was a problem for my binary search code).

    pair<bool,uint> match;  // index is undefined if false, don't check index in that case

    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    ar.insert_last( 111 );  // elemCnt=1
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last(  111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    ar.insert_last( 111 );  // elemCnt=2
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last(  111 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    ar.insert_last( 111 );  // elemCnt=3
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last(  111 ); if ( not match.first || match.second != 2 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    ar.insert_last( 111 );  // elemCnt=4
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last(  111 ); if ( not match.first || match.second != 3 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    ar.insert_last( 111 );  // elemCnt=5
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last(  111 ); if ( not match.first || match.second != 4 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    // Ok, now try inserting different values.
    ar.insert_last( 110 );  // elemCnt=6
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0+1 ) return Fail();  // 110 pushed
    match = ar.find_last(  111 ); if ( not match.first || match.second != 4+1 ) return Fail();  // everything up
    ar.insert_last( 112 );  // elemCnt=7
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0+1 ) return Fail();  // same index
    match = ar.find_last(  111 ); if ( not match.first || match.second != 4+1 ) return Fail();  // as 112 is after
    ar.insert_last( 109 );  // elemCnt=8
    match = ar.find_first( 111 ); if ( not match.first || match.second != 0+2 ) return Fail();  // 109 pushed
    match = ar.find_last(  111 ); if ( not match.first || match.second != 4+2 ) return Fail();  // everything up
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    // Now try a second equal range.
    ar.insert_last( 109 );  // elemCnt=9  // 109 again
    match = ar.find_first( 109 ); if ( not match.first || match.second != 0 ) return Fail();
    match = ar.find_last(  109 ); if ( not match.first || match.second != 1 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

    // Now try a equal range at the end.
    ar.insert_last( 555 );  // elemCnt=10
    match = ar.find_first( 555 ); if ( not match.first || match.second != 9 ) return Fail();
    match = ar.find_last(  555 ); if ( not match.first || match.second != 9 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();
    ar.insert_last( 555 );  // elemCnt=11
    match = ar.find_first( 555 ); if ( not match.first || match.second != 9 ) return Fail();
    match = ar.find_last(  555 ); if ( not match.first || match.second != 10 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();
    ar.insert_last( 555 );  // elemCnt=12
    match = ar.find_first( 555 ); if ( not match.first || match.second != 9 ) return Fail();
    match = ar.find_last(  555 ); if ( not match.first || match.second != 11 ) return Fail();
    match = ar.find_first( 0xdead ); if ( match.first ) return Fail();
    match = ar.find_last(  0xdead ); if ( match.first ) return Fail();

	CDEBUG << "ok \n";
 }

    return true;
}

#if COMPILE_TEST_ARRAY
// -- DEPRECATED --

/*****************************************************************************
 * Test TempArray class.
 *****************************************************************************/

// Must have external linkage to be a template parm.
const uint gTempArrSize = 100 * THOUSAND; 
int gTempArr[gTempArrSize];

bool
TestTempArray( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing TempArray class. \n";
	CDEBUG << "============================================\n";
 {
	typedef TempArray<int,gTempArr,gTempArrSize> TestArray;

	TestArray arr;
	if ( not arr.empty() ) return Fail();
	if ( arr.size() != 0 ) return Fail();

	arr.push_back( 0xabcdef );
	if ( arr[0] != 0xabcdef ) return Fail();
	if ( arr.empty() ) return Fail();
	if ( arr.size() != 1 ) return Fail();

	arr.resize( 100 );
	if ( arr.size() != 100 ) return Fail();

	CDEBUG << "ok \n";
 }


 {
	typedef TempArray<int,gTempArr,gTempArrSize> TestArray;

	TestArray arr( 50 );
	if ( arr.empty() ) return Fail();
	if ( arr.size() != 50 ) return Fail();
 }
	return true;
}

#endif // COMPILE_TEST_ARRAY

/*****************************************************************************
 * Test Bits class.
 *****************************************************************************/
bool
TestBits( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing Bits class. \n";
	CDEBUG << "============================================\n";
 {
    Bits bits;
    if ( bits != 0 ) return Fail();  // should be zeros

    bits.Set( 0 );
    if ( bits != uint(1<<0) ) return Fail();
    if ( not bits.Get( 0 ) ) return Fail();

    bits.Set( 1 );
    if ( bits != uint((1<<0)|(1<<1)) ) return Fail();
    if ( not bits.Get( 1 ) ) return Fail();

    bits.Set( 31 );
    if ( bits != uint((1<<0)|(1<<1)|(1<<31)) ) return Fail();
    if ( not bits.Get( 31 ) ) return Fail();

    bits.Clear( 0 );
    if ( bits != uint((1<<1)|(1<<31)) ) return Fail();
    if ( bits.Get( 0 ) ) return Fail();

    bits = uint((1<<16)|(1<<24));
    if ( bits != uint((1<<16)|(1<<24)) ) return Fail();
    if ( not bits.Get( 16 ) ) return Fail();
    if ( not bits.Get( 24 ) ) return Fail();
    if ( bits.Get( 0 ) ) return Fail();
    if ( bits.Get( 1 ) ) return Fail();

    bits = 0;
    bits.Set( 4, true );
    if ( not bits.Get(4) || uint(bits) != 16 ) return Fail();
    bits.Set( 4, false );
    if ( bits.Get(4) || uint(bits) != 0 ) return Fail();
 }
    CDEBUG << "ok" << endl;
	return true;
}

/*****************************************************************************
 * Test Iterator class.
 *****************************************************************************/
bool
TestIterator( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing Iterator class. \n";
	CDEBUG << "============================================\n";
 {
	//....................
  {
	// vector is empty.
	vector<int> vec;
	int loops = 0;
	STLIterator<int,vector<int> > iter( vec );
	while ( iter )
	{
		Fail();  // should not be reached
		++loops;
	}
	if ( loops != 0 ) Fail();
  }
	//....................


	//....................
	// vec has 1 item.
  {
	vector<int> vec;
	vec.push_back( 123 );
	int loops = 0;
	STLIterator<int,vector<int> > iter( vec );
	while ( iter )
	{
		if ( loops == 0 )
		{
			int val = iter.Next();
			if ( val != 123 ) Fail();
		}
		else { Fail(); }

		++loops;
	}
	if ( loops != 1 ) Fail();
  }


	//....................
	// vec has 100 items.
  {
	vector<int> vec;
	for ( uint i = 0; i < 100; ++i ) vec.push_back( i );
	int loops = 0;
	STLIterator<int,vector<int> > iter( vec );
	while ( iter )
	{
		int val = iter.Next();
		if ( val != loops ) Fail();
		++loops;
	}
	if ( loops != 100 ) Fail();
  }

 }

	CDEBUG << "ok \n";
	return true;
}

/*****************************************************************************
 * Test misc.
 *****************************************************************************/
bool
TestMisc( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing misc. \n";
	CDEBUG << "============================================\n";
 {
	Milliseconds millisecs(THOUSAND);
	int t = int( Microseconds(millisecs) );
	if ( t != MILLION ) { Fail(); }
 }

 {
	Milliseconds millisecs( Seconds(1) );
	int t = int(millisecs);
	if ( t != THOUSAND ) { Fail(); }
 }

 {
	Microseconds microsecs( Seconds(1) );
	int t = int(microsecs);
	if ( t != MILLION ) { Fail(); }
 }

	CDEBUG << "ok \n";
	return true;
}

/*****************************************************************************
 * main.
 *****************************************************************************/
int main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	int res = 1;
	if ( TestArray()
      && TestTinyArray()
      && TestSmallArray()
      && TestSortedArray()
#if COMPILE_TEST_ARRAY
      && TestTempArray()  // DEPRECATED
#endif
      && TestBits()
      && TestIterator()
      && TestMisc()
       )
	{
		CDEBUG << "Test PASSED.\n";
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED! \n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
