/*****************************************************************************
 * $LastChangedDate: 2008-05-02 08:11:38 -0400 (Fri, 02 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of file functions: ReadFile(), FileSize(), RemoveFile(), etc.
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include <sstream>
#include "test.hh"
#include "base/module.hh"
#include "base/file.hh"
using namespace base;

/*****************************************************************************
 * Test.
 *****************************************************************************/
bool
Test( void )
{
	const string fname = "/tmp/test_file.dat";

	// Ensure dummy file doesn't exist yet.
//	if ( ! RemoveFile( fname ) ) return Fail();  // will normally return false
	RemoveFile( fname );  // ensure 
	if ( IfFileExists( fname ) ) return Fail();

	// Create dummy file.
	fstream fs;
	fs.open( fname.c_str(), ios::out|ios::binary|ios::trunc );
	if ( ! fs.good() || ! fs.is_open() ) return Fail();
	if ( not IfFileExists( fname ) ) return Fail();

	// Write dummy data to string stream.
	ostringstream os;
	for ( int i = 0; i < 100; ++i ) os << "abc123\n";

	// Write to dummy file.
	//fs << os;
	fs << os.str();
	if ( ! fs.good() || ! fs.is_open() ) return Fail();

	// Close dummy file.
	fs.flush();
	fs.close();

	// Read the file.
	string ibuf;
	if ( ! ReadFile( fname, ibuf ) ) return Fail();
	if ( ibuf != os.str() ) return Fail();
	if ( ibuf.size() == 0 || os.str().size() == 0 ) return Fail();

	// Now try error cases (missing file).
	if ( ReadFile( "MISSING_FILE", ibuf ) ) return Fail();
	// Skip FileSize(FILE*) as fopen("MISSING_FILE") would return NULL (core dump).

	// Cleanup.
	if ( ! RemoveFile( fname ) ) return Fail();
	if ( IfFileExists( fname ) ) return Fail();

	return true;
}

/*****************************************************************************
 * Test misc file functions.
 *****************************************************************************/
bool
Test2( void )
{
// RemoveSuffix()
	{
		string fname = "test.txt";
		RemoveSuffix( fname, ".txt" );
		if ( fname != "test" ) return Fail();
	}

	{
		string fname = "test.txt";
		RemoveSuffix( fname, ".MISSING" );  // should be a NOP
		if ( fname != "test.txt" ) return Fail();
	}

// IfAbsPath()
	{
		string fname = "test.txt";
		if ( IfAbsPath(fname) ) return Fail();
	}

	{
		string fname = "/test.txt";
		if ( not IfAbsPath(fname) ) return Fail();
	}

	return true;
}

/*****************************************************************************
 * main.
 *****************************************************************************/
int
main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	int res = 1;
	if ( Test() and Test2() )
	{
		CDEBUG << "Test PASSED. \n";
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED! \n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
