/*****************************************************************************
 * $LastChangedDate: 2008-09-01 11:11:25 -0400 (Mon, 01 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of basic functons (base/funcs*.hh).
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "base/funcs_stl.hh"

/*****************************************************************************
 * Test IfDuplicate().
 *****************************************************************************/
bool
TestIfDuplicate( void )
{
 {
	vector<int> v1;
	vector<int> v2;
	v1.push_back( 1 );
	v2.push_back( 2 );
	if ( IfDuplicate( v1, v2 ) ) return Fail();
 }

 {
	vector<int> v1;
	vector<int> v2;
	v1.push_back( 1 );
	v1.push_back( 2 );
	v2.push_back( 2 );
	v2.push_back( 3 );
	if ( not IfDuplicate( v1, v2 ) ) return Fail();
 }

	return true;
}

/*****************************************************************************
 * Test IfSameSign().
 *****************************************************************************/
bool
TestIfSameSign( void )
{
	if ( not IfSameSign( 1, 1 ) ) return Fail();
	if ( not IfSameSign( -1, -1 ) ) return Fail();
	if ( IfSameSign( 1, -1 ) ) return Fail();
	if ( IfSameSign( 0, -1 ) ) return Fail();
	if ( not IfSameSign( 0, 0 ) ) return Fail();
	return true;
}

/*****************************************************************************
 * Test NEG().
 *****************************************************************************/
bool
TestNEG( void )
{
	if ( NEG(1)  != -1 ) return Fail();
	if ( NEG(-1) != -1 ) return Fail();
	if ( NEG(0) != 0 ) return Fail();
	return true;
}

/*****************************************************************************
 * Test Min()/Max().
 *****************************************************************************/
bool
TestMinMax( void )
{
	if ( Min(10,20) != 10 ) return Fail();
	if ( Min(20,10) != 10 ) return Fail();
	if ( Min(20,-10) != -10 ) return Fail();

	if ( MIN(10,20) != 10 ) return Fail();
	if ( MIN(20,10) != 10 ) return Fail();
	if ( MIN(20,-10) != -10 ) return Fail();

	if ( Max(10,20) != 20 ) return Fail();
	if ( Max(20,10) != 20 ) return Fail();
	if ( Max(20,-10) != 20 ) return Fail();

	if ( MAX(10,20) != 20 ) return Fail();
	if ( MAX(20,10) != 20 ) return Fail();
	if ( MAX(20,-10) != 20 ) return Fail();

	return true;
}

/*****************************************************************************
 * Test IfPow2().
 *****************************************************************************/
bool
TestIfPow2( void )
{
	if ( IfPow2(0) ) return Fail();      // zero isn't a power-of-2
	if ( not IfPow2(1) ) return Fail();  // 2^0 == 1
	if ( not IfPow2(2) ) return Fail();
	if ( IfPow2(3) ) return Fail();
	if ( not IfPow2(4) ) return Fail();
	if ( IfPow2(5) ) return Fail();
	if ( not IfPow2(8) ) return Fail();
	if ( not IfPow2(256) ) return Fail();
	return true;
}

/*****************************************************************************
 * Test Clamp().
 *****************************************************************************/
bool
TestClamp( void )
{
	// Test Clamp(int i, int limit)
	if ( Clamp(50,50) != 50 ) return Fail();
	if ( Clamp(51,50) != 50 ) return Fail();
	if ( Clamp(60,50) != 50 ) return Fail();
	if ( Clamp(-60,50) != 0 ) return Fail();
	if ( Clamp(40,50) != 40 ) return Fail();

	// Test uint8 ClampFF(uint i)
	if ( ClampFF( 1234u ) != 0xff ) return Fail();
	if ( ClampFF( 1u ) != 1 ) return Fail();
	if ( ClampFF( 0xff ) != 0xff ) return Fail();
	if ( ClampFF( 0x100 ) != 0xff ) return Fail();

	// Test uint8 ClampFF(int i).
	if ( ClampFF( -50 ) != 0 ) return Fail();
	if ( ClampFF( 255 ) != 255 ) return Fail();
	if ( ClampFF( 256 ) != 255 ) return Fail();

	// Test ClampFF(fp).
	if ( ClampFF( -50.5f ) != 0 ) return Fail();
	if ( ClampFF( 254.90f ) != 255 ) return Fail();
	if ( ClampFF( 256.10f ) != 255 ) return Fail();

	// Test Clamp1(fp).
	if ( Clamp1( -123.0f ) != 0.0f ) return Fail();
	if ( Clamp1( 1.5f ) != 1.0f ) return Fail();
	if ( Clamp1( 0.5f ) != 0.5f ) return Fail();

	return true;
}

/*****************************************************************************
 * main.
 *****************************************************************************/
int main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	int res = 1;
	if ( TestIfDuplicate()
     and TestIfSameSign()
     and TestNEG()
     and TestMinMax()
     and TestIfPow2()
     and TestClamp()
       )
	{
		CDEBUG << "Test PASSED.\n";
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED! \n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
