/*****************************************************************************
 * $LastChangedDate: 2008-10-12 13:07:13 -0400 (Sun, 12 Oct 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of Lua interface.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "base/module.hh"
#include "base/file.hh"
using namespace base;
#include "lua_bind/module.hh"
using namespace lua;

namespace lua_bind {

// Interface for this test.
extern bool gLuaTestObjectWhileHoldingRefCnt;

int GetTestClassInstanceCnt( void );
int GetTestClassInstanceCntNeverDec( void );

// KLUDGE/WORKAROUND for linker since liblua_bind_test.a omits simulator libs.
void RegisterCmdLine( lua_State* L, int argc, char** argv ) { }
void RegisterMath( lua_State* L ) { }
void RegisterProgram( lua_State* L ) { }
void RegisterPhysics( lua_State* L ) { }
void RegisterAircraftPhysics( lua_State* L ) { }
void RegisterObject( lua_State* L ) { }
void RegisterDyna( lua_State* L ) { }
void RegisterAircraft( lua_State* L ) { }
void RegisterCarrier( lua_State* L ) { }
void RegisterKeyboard( lua_State* L ) { }
void RegisterJoystick( lua_State* L ) { }
void RegisterSound( lua_State* L ) { }
void RegisterView( lua_State* L ) { }
void RegisterViewpoint( lua_State* L ) { }
void RegisterChaseViewpoint( lua_State* L ) { }
void RegisterWindow( lua_State* L ) { }
void RegisterHudText( lua_State* L ) { }
void RegisterHudBox( lua_State* L ) { }
void RegisterGui( lua_State* L ) { }
void RegisterWorld( lua_State* L ) { }
void RegisterControl( lua_State* L ) { }
void RegisterFX( lua_State* L ) { }
void RegisterSky( lua_State* L ) { }
void RegisterConfBin( lua_State* L ) { }
void RegisterReplay( lua_State* L ) { }
void RegisterGameHud( lua_State* L ) { }
} // namespace lua_bind

using namespace lua_bind;

/*****************************************************************************
 *
 *****************************************************************************/
bool
TestLua( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing Lua singleton. \n";
	CDEBUG << "============================================\n";

	// Pathname passed to RunScript() is implicitly catenated
	// with the directory containing Lua scripts.

	//..........................................................................
	// Test running simple script.
	try
	{
		GET_LUA().RunScript( "tests/simple.lua" );  // palomino/scripts/tests
		GET_LUA().RunScript( "tests/simple.lua" );  // again to test caching bytecode
		GET_LUA().RunScript( "tests/simple.lua" );  // again to test caching bytecode
	}
	catch ( const Exception& e )
	{
		CERROR << e.GetText() << endl;
		return Fail();
	}

	//..........................................................................
	// Test trying to run a missing script.
	try
	{
		GET_LUA().RunScript( "MISSING_SCRIPT.lua" );
		return Fail();  // should've thrown
	}
	catch ( const Exception& e )
	{
		CERROR << "Passed trying to run missing Lua script" << endl;
	}

	//..........................................................................
	// Run the scripts containing the test cases.
	try
	{
		GET_LUA().RunScript( "tests/bools.lua" );
		GET_LUA().RunScript( "tests/numbers.lua" );
		GET_LUA().RunScript( "tests/strings.lua" );
		GET_LUA().RunScript( "tests/tables.lua" );
		GET_LUA().RunScript( "tests/vec.lua" );
	}
	catch ( const Exception& e )
	{
		CERROR << e.GetText() << endl;
		return Fail();
	}


	//..........................................................................
	// Test Lua::ReadVarAsBool().
	try
	{
		if ( not GET_LUA().ReadVarAsBool( "bool_true" )
          or     GET_LUA().ReadVarAsBool( "bool_false" )
          or not GET_LUA().ReadVarAsBool( "bool_1" )
          or     GET_LUA().ReadVarAsBool( "bool_0" ) )
		{
			return Fail();
		}
	}
	catch ( const Exception& e )
	{
		CERROR << e.GetText() << endl;
		return Fail();
	}

	//..........................................................................
	// Test Lua::ReadVarAsNumber().
	try
	{
		// Test the Lua developer's assertion that float round-off doesn't occur.
		if ( int(GET_LUA().ReadVarAsNumber( "var123" ) != 123 )
          or int(GET_LUA().ReadVarAsNumber( "varNeg456" ) != -456 ) )
		{
			return Fail();
		}

		// Test trying to read a missing variable.
		try
		{
			GET_LUA().ReadVarAsNumber( "MISSING" );
			return Fail();
		}
		catch(...)
		{
			// ok
		}
	}
	catch ( const Exception& e )
	{
		return Fail();
	}

	//..........................................................................
	// Test Lua::ReadVarAsString().
	try
	{
		// Test the Lua developer's assertion that float round-off doesn't occur.
		if ( GET_LUA().ReadVarAsString( "string_ABC" ) != "ABC"
          or GET_LUA().ReadVarAsString( "string_789" ) != "789"
          or GET_LUA().ReadVarAsString( "string_0123XYZ" ) != "0123XYZ"
          or GET_LUA().ReadVarAsString( "string_empty" ) != "" )
		{
			return Fail();
		}

		// Test trying to read a missing variable.
		try
		{
			GET_LUA().ReadVarAsString( "MISSING" );
			return Fail();
		}
		catch(...)
		{
			// ok
		}
	}
	catch ( const Exception& e )
	{
		return Fail();
	}

	//..........................................................................
	// Test Lua::ReadTableAsNumber() [array variant].
	try
	{
		// "table_numbers" using integer indexs.
		if ( GET_LUA().ReadTableAsNumber( "table_numbers", 1 ) != 111 )
		{
			return Fail();
		}

		if ( GET_LUA().ReadTableAsNumber( "table_numbers", 2 ) != 222 )
		{
			return Fail();
		}

		// This element is missing so an exception should be thrown.
		try
		{
			GET_LUA().ReadTableAsString( "table_numbers", 9999 );
			return Fail();
		}
		catch(...)
		{
			// ok
		}
	}
	catch ( const Exception& e )
	{
		return Fail();
	}

	//..........................................................................
	// Test Lua::ReadTableAsNumber() [string key variant].
	try
	{
		// "table_numbers_key" using integer indexs.
		if ( GET_LUA().ReadTableAsNumber( "table_numbers_key", "k1" ) != 111 )
		{
			return Fail();
		}

		if ( GET_LUA().ReadTableAsNumber( "table_numbers_key", "k2" ) != 222 )
		{
			return Fail();
		}

		// This element is missing so an exception should be thrown.
		try
		{
			GET_LUA().ReadTableAsString( "table_numbers_key", "9999" );
			return Fail();
		}
		catch(...)
		{
			// ok
		}
	}
	catch ( const Exception& e )
	{
		return Fail();
	}

	//..........................................................................
	// Test Lua::ReadTableAsString() [array variant].
	try
	{
		// "table_array" using integer indexs.
		if ( GET_LUA().ReadTableAsString( "table_array", 1 ) != "elem1" )
		{
			return Fail();
		}

		if ( GET_LUA().ReadTableAsString( "table_array", 2 ) != "elem2" )
		{
			return Fail();
		}

		// This element is missing so an exception should be thrown.
		try
		{
			GET_LUA().ReadTableAsString( "table_array", 3 );
			return Fail();
		}
		catch(...)
		{
			// ok
		}
	}
	catch ( const Exception& e )
	{
		return Fail();
	}

	//..........................................................................
	// Test Lua::ReadTableAsString() [key string variant].
	try
	{
		if ( GET_LUA().ReadTableAsString( "table_string", "a" ) != "A" )
		{
			return Fail();
		}

		if ( GET_LUA().ReadTableAsString( "table_string", "b" ) != "B" )
		{
			return Fail();
		}

		// This element is missing so an exception should be thrown.
		try
		{
			GET_LUA().ReadTableAsString( "table_string", "" );
			return Fail();
		}
		catch(...)
		{
			// ok
		}
	}
	catch ( const Exception& e )
	{
		return Fail();
	}

	return true;
}

/*****************************************************************************
 * Test the exported C++ class Test (lua_bind/bind_test.hh).
 *****************************************************************************/
bool
TestCppClass( void )
{
	CDEBUG << "============================================\n";
	CDEBUG << "Testing the exported C++ class \"Test\". \n";
	CDEBUG << "============================================\n";

	//..........................................................................
	// Run Lua script to test the exported C++ class.
	try
	{
		CDEBUG << "Testing that garbage-collector destroys C++ object...\n";

		GET_LUA().RunScript( "tests/class_Test.lua" );

		if ( GET_LUA().ReadVarAsString( "test_GetABC" ) != string("ABC") )
		{
			return Fail();
		}

		// Test that the C++ objects was created exactly once.
		if ( GetTestClassInstanceCntNeverDec() != 1 )
		{ return Fail(); }

		// Test that Lua's garbage-collector indirectly
		// caused the C++ object to be destroyed.
		if ( GetTestClassInstanceCnt() != 0 )
		{ return Fail(); }
	}
	catch ( const Exception& e )
	{
		CERROR << e.GetText() << endl;
		return Fail();
	}

	//..........................................................................
	// Now test when a reference is being held.
	// See bind_test.cc.
	try
	{
		CDEBUG << "Testing garbage-collector lets C++ object survive when C++ holds a reference\n";

		// -- See lua_bind/bind_test.cc --
		gLuaTestObjectWhileHoldingRefCnt = true;

		GET_LUA().RunScript( "tests/class_Test.lua" );

		if ( GET_LUA().ReadVarAsString( "test_GetABC" ) != string("ABC") )
		{
			return Fail();
		}

		// Ok, this C++ object was instantiated twice,
		// and the second instance should still exist,
		// even though the Lua script called collectgarbage().

		if ( GetTestClassInstanceCntNeverDec() != 2 )
		{ return Fail(); }

		// Test that Lua's garbage-collector indirectly
		// caused the C++ object to be destroyed.
		if ( GetTestClassInstanceCnt() != 1 )
		{ return Fail(); }
	}
	catch ( const Exception& e )
	{
		CERROR << e.GetText() << endl;
		return Fail();
	}

	return true;
}

/*****************************************************************************
 * main.
 *****************************************************************************/
int main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;

	// Find the script dir.
	string scriptDir;
	if ( IfDirExists( "scripts" ) )
	{
		GET_LUA().SetScriptDir( "scripts" );
	}
	else if ( IfDirExists( "../scripts" ) )
	{
		GET_LUA().SetScriptDir( "../scripts" );
	}
	else if ( IfDirExists( "../../scripts" ) )
	{
		GET_LUA().SetScriptDir( "../../scripts" );
	}
	else
	{
		CERROR << "ERROR: cannot find Lua scripts directory" << endl;
		EXIT( 1 );
	}

	int res = 1;
	if ( TestLua()
     and TestCppClass() )
	{
		CDEBUG << "Test PASSED.\n";
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED! \n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
