/*****************************************************************************
 * $LastChangedDate: 2008-07-05 10:47:33 -0400 (Sat, 05 Jul 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test SinCos().
 *
 * This is a bare non-automated test that is x86-specific.
 * It is excluded from makefile and run_tests script.
 *
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include <iostream>
#include <cmath>
using namespace std;
#include "test.hh"

#define CPU_X86 1
#if __POWERPC__
#undef CPU_X86
#define CPU_PPC 1
#endif

typedef float Radian;
const double PI  = 3.14159265358979323846;
const double PI2 = 6.28318530717958647692;   // pi*2

#if defined(__GNUC__) && defined(CPU_X86)
template<typename FP>
pair<FP,FP>
SinCos_asm( const Radian rad )
{
	pair<FP,FP> si_co;
	asm("fsincos" : "=t" (si_co.second), "=u" (si_co.first) : "0" (rad));
	return si_co;
}
#endif

#if defined(__GNUC__) && _GLIBCXX_HAVE_SINCOS
template<typename FP>
pair<FP,FP>
SinCos_gnu( const Radian rad )
{
	pair<double,double> si_co;  // must be specific in this case
	sincos( rad, &si_co.first, &si_co.second );  // GNU extension
	return make_pair<FP,FP>( si_co.first, si_co.second );
}
#endif

template<typename FP>
pair<FP,FP>
SinCos_sin_cos( const Radian rad )
{
	pair<FP,FP> si_co;
	si_co.first  = sin( rad );
	si_co.second = cos( rad );
	return si_co;
}

void Test( const Radian rad )
{
	COUT << "------ rad = " << rad << "--------------" << endl;
#if defined(__GNUC__) && defined(CPU_X86)
  {
	pair<float,float> si_co = SinCos_asm<float>( rad );
    COUT << "SinCos_asm_x86: " << si_co.first << " " << si_co.second << endl;
  }
#endif

#if defined(__GNUC__) && _GLIBCXX_HAVE_SINCOS
  {
	pair<float,float> si_co = SinCos_gnu<float>( rad );
    COUT << "SinCos_gnu    : " << si_co.first << " " << si_co.second << endl;
  }
#endif

  {
	pair<float,float> si_co = SinCos_sin_cos<float>( rad );
    COUT << "SinCos        : " << si_co.first << " " << si_co.second << endl;
  }
}

int main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	Test( 0.25 );
	Test( 0.5 );
	Test( M_PI );
	Test( -M_PI );
	Test( M_PI/2 );
	Test( -M_PI/2 );
	Test( M_PI/4 );
	Test( -M_PI/4 );
	TEST_MAIN_END;
	exit(0);
}
