#!/bin/sh
# $LastChangedDate: 2008-10-13 17:40:53 -0400 (Mon, 13 Oct 2008) $
# Author: Jim Brooks  http://www.palomino3d.org
# Do some checks before compiling.
#-------------------------------------------------------------------------------

TOP_SRC_DIR=$1
if [ "$TOP_SRC_DIR" = "" ]; then
    TOP_SRC_DIR=src
fi
[ -d $TOP_SRC_DIR ] || exit 1

LUA_DIR=scripts
[ -d $LUA_DIR ] || exit 1

# lwlint available?
if ( which lwlint >/dev/null 2>&1 ); then
	echo "checks.sh: will use lwlint."
    HAVE_LWLINT=1
fi

Checks()
{
    SRC_DIR=$1

	# Run lwlint if available.
	if [ $HAVE_LWLINT ]; then
		if ( ! lwlint ${SRC_DIR}/*.cc ${SRC_DIR}/*.hh ); then
			echo "ERROR: lwlint found errors." >/dev/stderr
			exit 1
		fi
	fi

    # "namespace global" singular is correct (minor).
    if ( grep -i "namespace globals" $SRC_DIR/*.* ); then
        echo "ERROR: namespace globals should be singular (minor)" >/dev/stderr
        exit 1
    fi

    # Catch: iter < end()
    if ( grep -i "iter.* < .*end[(]" $SRC_DIR/*.* ); then
        echo "ERROR: iter < end() is wrong!" >/dev/stderr
        exit 1
    fi

    # A habitual mistake that causes a buffer overrun is passing sizeof()
    # to InitArray() which expects an element count, not a byte count.
    if ( grep "InitArray.*sizeof" $SRC_DIR/*.* ); then  # not grep -q
        echo "ERROR: sizeof() cannot be passed to InitArray()" >/dev/stderr
        exit 1
    fi

    # Catch writing #if DEBUG2.  #if DEBUG == 2 is correct.
    if ( grep "if.*DEBUG2" $SRC_DIR/*.* ); then
        echo "ERROR: should be DEBUG==2" >/dev/stderr
        exit 1
    fi

    # Catch this mistake THREAD_CODE( Atomic::Add() )
    # because when compiled single-thread it expands nothing.
    if ( grep "\(^\|[^/]\)THREAD_CODE.*Atomic" $SRC_DIR/*.* ); then
        echo "ERROR: THREAD_CODE( Atomic ) expands nothing if ! COMPILE_THREADS" >/dev/stderr
        exit 1
    fi

    # Should be COMPILE_THREADS (plural).
    if ( grep "if COMPILE_THREAD$" $SRC_DIR/*.* ) || ( grep "if COMPILE_THREAD " $SRC_DIR/*.* ); then
        echo "ERROR: Should be COMPILE_THREADS (plural)." >/dev/stderr
        exit 1
    fi

    # SharedPtr shouldn't be a reference.
    FILES=
    for FILE in $SRC_DIR/*.*
    do
        if ( echo $FILE | grep -q -v shared_ptr ); then
            FILES="$FILES $FILE"
        fi

        if ( echo $FILE | grep -q -v shared_ptr ) && ( grep "shptr<.*>&" $FILES ); then
            echo "ERROR: shptr shouldn't be a reference" >/dev/stderr
            exit 1
        fi

        if ( echo $FILE | grep -q -v shared_ptr ) && ( grep "RefPtr<.*>&" $FILES ); then
            echo "ERROR: RefPtr shouldn't be a reference" >/dev/stderr
            exit 1
        fi

    done
}

# For each subdir of SRC_DIR.
for SUBDIR in $TOP_SRC_DIR/*;
do
    if [ -d $SUBDIR ]; then
		if ( echo $SUBDIR | grep -q "deprecated\|lua_lang" ); then
			:
		else
			Checks $SUBDIR
		fi
    fi
done

# Check Lua code.
if ( grep -n self\\.[a-zA-Z_]*\( $LUA_DIR/*.lua ); then
    echo "ERROR: self.Method() should be self:Method()" >/dev/stderr
    exit 1
fi

echo "Checks passed (ok)."
exit 0
